﻿import sys
import clr
import re
import System
import time
clr.AddReferenceToFile("DotPUniGenEngine.dll")
from tfm.DotPUniGen.Engine import *
from tfm.Common import *

UserName = 'Xander9009'
CommunityWad = True

sN = System.Environment.NewLine
lDefault = Language.English

# X99: I've added this to more easily test if a given ability will be coded correctly.
# TFM's test feature (in the Tools menu) will cause a basic card output with all of these abilities coded as they would have been normally.
# Please note that if needed, the test card's name will be Some Card.
# Ctrl-f for "def Test" if you need to change stats, types, or other info.
lstTestAbilities = [
	"Embalm {1}",
	"Investigate."
	]

# See bottom for other information.

def Info():
	return """DotP2014 Output
v1.13.1
by thefiremind | modified by Xander9009
Generates a DotP2014-compatible XML string from card information"""

def Purpose():
	return PluginPurpose.Output

def Get(crdCard, sPrefix):
	sXml = '<?xml version="1.0"?>' + sN + '<CARD_V2 ExportVersion="1">' + sN
	# compute missing information
	sCardName = StringUtils.Nameify(crdCard.GetTitle(Language.English))
	sPrefixedId = crdCard.MultiverseID
	if len(sPrefix) > 0:
		while len(sPrefixedId) < 6:
			sPrefixedId = '0' + sPrefixedId
		sPrefixedId = sPrefix + sPrefixedId
	if CommunityWad:
		crdCard.FileName = sCardName + '_CW'
	else:
		crdCard.FileName = sCardName + '_' + sPrefixedId
	# add filename
	sXml += SimpleNode('FILENAME', 'text', crdCard.FileName)
	# add cardname
	sXml += SimpleNode('CARDNAME', 'text', sCardName)
	# add title
	sXml += ('	<TITLE>' + sN)
	for lLang in MiscUtils.GetValues[Language]():
		sTitle = crdCard.GetTitle(lLang)
		if len(sTitle) < 1:
			sTitle = crdCard.GetTitle(lDefault)
		sXml += LocalizedTextNode(lLang, sTitle)
	sXml += ('	</TITLE>' + sN)
	# add prefixed multiverse id
	sXml += SimpleNode('MULTIVERSEID', 'value', sPrefixedId)
	# add prefixed multiverse id as art id as well
	if CommunityWad:
		sXml += SimpleNode('ARTID', 'value', sCardName)
	else:
		sXml += SimpleNode('ARTID', 'value', sPrefixedId)
	# add colors if necessary
	if crdCard.ForcedColors:
		sColors = ""
		for ccColor in MiscUtils.GetValues[CardColor]():
			if ccColor != CardColor.Colorless and crdCard.HasColor(ccColor):
				sColors += Constants.ColorToCode(ccColor)
		if len(sColors) < 1:
			sColors = "C"
		sXml += SimpleNode('COLOUR', 'value', sColors)
	# add artist
	sXml += SimpleNode('ARTIST', 'name', crdCard.Artist.replace("&", "&amp;"))
	# add casting cost
	sXml += SimpleNode('CASTING_COST', 'cost', crdCard.CastingCost.Replace("{C}", "{1}"))
	# add flavor if present
	sDefaultFlavor = crdCard.GetFlavor(lDefault)
	if len(sDefaultFlavor) > 0:
		sXml += ('	<FLAVOURTEXT>' + sN)
		for lLang in MiscUtils.GetValues[Language]():
			sFlavor = crdCard.GetFlavor(lLang)
			if len(sFlavor) < 1:
				sFlavor = sDefaultFlavor
			sXml += LocalizedTextNode(lLang, sFlavor)
		sXml += ('	</FLAVOURTEXT>' + sN)
	# add supertypes
	for sSupertype in crdCard.GetSupertypesArray():
		sXml += SimpleNode('SUPERTYPE', 'metaname', sSupertype)
	# add types
	for sType in crdCard.GetTypesArray():
		sXml += SimpleNode('TYPE', 'metaname', sType)
	# add sub-types
	for sSubType in crdCard.GetSubTypesArray():
		sXml += SimpleNode('SUB_TYPE', 'metaname', sSubType)
	# add expansion
	sXml += SimpleNode('EXPANSION', 'value', crdCard.Expansion)
	# add rarity
	sRarity = crdCard.GetRarity().ToString()[:1]
	if sRarity == "S":
		# special rarity would crash the game, so we change it into something that works but is also easy to notice
		sRarity = "T"
	sXml += SimpleNode('RARITY', 'metaname', sRarity)
	# add P/T if present
	bGotPower = False
	if len(crdCard.Power) > 0:
		bGotPower = True
		sXml += SimpleNode('POWER', 'value', crdCard.Power)
		sXml += SimpleNode('TOUGHNESS', 'value', crdCard.Toughness)
	sXml += AbilitiesAndHelp(crdCard)
	sXml = sXml.replace("CARDNAMETAG", sCardName)
	moMatches = re.finditer('CW_Tokens(?:_Tapped|)\("([A-Z0-9_]*)"', sXml)
	for moMatch in moMatches:
		if moMatch:
			for iIndex in range(1, len(moMatch.groups())+1):
				sTokenName = str(moMatch.group(iIndex))
				if sTokenName[0:7] == "EMBALM_" or sTokenName[0:11] == "ETERNALIZE_":
					sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '" />' + sN
				else:
					sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_1" />' + sN
					if str(moMatch.group(iIndex)) == "ELDRAZI_SCION_C_1_1_C_S_MM":
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_2" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_3" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_C" />' + sN
					elif str(moMatch.group(iIndex)) == "CLUE_A_C_S":
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_2" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_3" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_4" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_5" />' + sN
						sXml += '	<TOKEN_REGISTRATION reservation="1" type="TOKEN_' + sTokenName + '_CW_6" />' + sN
	# Correct hybrid mana and other special characters. Do not affect lines with localized text, though.
	for sShortCode, sLongCode in dicManaShortLongCodes.iteritems():
		sXml = re.sub('(?<!LOCALISED_TEXT.*){' + sShortCode + '}', '{' + sLongCode + '}', sXml)
	#Correct plus signs the game won't understand. This is much easier than other methods.
	sXml = re.sub('Power_Add\(+', 'Power_Add(', sXml)
	sXml = re.sub('Tougness_Add\(+', 'Toughness_Add(', sXml)
	# add the default combat SFX nodes if P/T were added before
	if bGotPower:
		sXml += ('	<SFX text="COMBAT_BLUNT_LARGE_ATTACK" power_boundary_min="4" power_boundary_max="-1" />' + sN)
		sXml += ('	<SFX text="COMBAT_BLUNT_SMALL_ATTACK" power_boundary_min="1" power_boundary_max="3" />' + sN)
	if UserName:
		sXml += ('	<AUTHOR><![CDATA[' + UserName + ']]></AUTHOR>' + sN)
		sXml += ('	<EDITORS><![CDATA[' + UserName + ']]></EDITORS>' + sN)
		sXml += ('	<DATE><![CDATA[' + time.strftime("%d-%m-%y") + ']]></DATE>' + sN)
	elif CommunityWad:
		sXml += ('	<AUTHOR><![CDATA[thefiremind, Xander9009]]></AUTHOR>' + sN)
		sXml += ('	<EDITORS><![CDATA[thefiremind, Xander9009]]></EDITORS>' + sN)
		sXml += ('	<DATE><![CDATA[' + time.strftime("%d-%m-%y") + ', ' + time.strftime("%d-%m-%y") + ']]></DATE>' + sN)
	sXml += '</CARD_V2>'
	if re.search('<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_', sXml):
		sOldName = crdCard.FileName
		crdCard.FileName = re.sub('_CW_', '_MM_CW_', crdCard.FileName)
		sXml = re.sub(sOldName,crdCard.FileName, sXml)
	if CommunityWad and crdCard.ForcedColors and not crdCard.CastingCost:
		sXml = re.sub(crdCard.FileName, '_' + crdCard.FileName, sXml)
		crdCard.FileName = '_' + crdCard.FileName
	return sXml

def AbilitiesAndHelp(crdCard):
	sXml = ""
	lstAbilityNodes = []
	lstHelpNodes = []
	# Process each ability
	for iIndex in range(0, crdCard.GetAbilitiesCount()):
		tupAH = ProcessAbility(crdCard, iIndex)
		lstAbilityNodes.append(tupAH[0])
		if len(tupAH[1]) > 0:
			lstHelpNodes.append(tupAH[1])
	# Add code dependent on which abilities were added such as backup mana abilities and token creation.
	sCanProduce = ""
	sDefaultColours = ""
	bSimple = False
	bComplex = False
	for sAbility in lstAbilityNodes:
		moManaMatch = re.search('<!--(COMPLEX|SIMPLE)MANAPRODUCTION-((?:(?:[WUBRGC]|[0-9]+)+-)+)->', sAbility)
		if moManaMatch:
			bSimple = str(moManaMatch.group(1)) == 'SIMPLE'
			bComplex = str(moManaMatch.group(1)) == 'COMPLEX'
			sGroup = moManaMatch.group(2)
			for sString in sGroup.replace('-', ' ').split():
				sCanProduce += sString
				if not re.search('<COST (?!type="TapSelf" />)', sAbility):
					sDefaultColours += sString + '-'
			sAbility = re.sub('<ACTIVATED_ABILITY>', '<ACTIVATED_ABILITY forced_skip="1">', sAbility)
		sXml += sAbility
	if sCanProduce:
		bC = re.search('[0-9]', sCanProduce) or 'C' in sCanProduce
		bW = 'W' in sCanProduce
		bU = 'U' in sCanProduce
		bB = 'B' in sCanProduce
		bR = 'R' in sCanProduce
		bG = 'G' in sCanProduce
		sRegistrations = ""
		if bC:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_C" />' + sN
		if bW:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_W" />' + sN
		if bU:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_U" />' + sN
		if bB:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_B" />' + sN
		if bR:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_R" />' + sN
		if bG:
			sRegistrations += '	<TOKEN_REGISTRATION reservation="1" type="RSN_TOKEN_MANA_G" />' + sN
		sCanProduce = ""
		if bC:
			sCanProduce += "{1}"
		if bW:
			sCanProduce += "{W}"
		if bU:
			sCanProduce += "{U}"
		if bB:
			sCanProduce += "{B}"
		if bR:
			sCanProduce += "{R}"
		if bG:
			sCanProduce += "{G}"
		sManaAbilities = '	<STATIC_ABILITY>' + sN + '		<CONTINUOUS_ACTION layer="0">' + sN + '			RSN_ClearCanProduceMana()' + sN + '			RSN_MarkCanProduceMana("' + sCanProduce + '")' + sN + '		</CONTINUOUS_ACTION>' + sN
		sDefaultColour = ""
		if sDefaultColours and (bSimple is not bComplex):
			for sMana in sDefaultColours.replace('-', ' ').split():
				if not sDefaultColour:
					if sMana == 'C' or re.search('[0-9]', sMana):
						sDefaultColour = 'COLOURLESS'
					if sMana == 'W':
						sDefaultColour = 'WHITE'
					if sMana == 'U':
						sDefaultColour = 'BLUE'
					if sMana == 'B':
						sDefaultColour = 'BLACK'
					if sMana == 'R':
						sDefaultColour = 'RED'
					if sMana == 'G':
						sDefaultColour = 'GREEN'
		if sDefaultColour and bSimple:
			sManaAbilities += '		<CONTINUOUS_ACTION layer="8">' + sN + '			local nDefaultColour = COLOUR_' + sDefaultColour + sN + '			local oCard = EffectSource()' + sN + '			if (RSN_CheckSwitchToFallback(oCard)) then' + sN + '				local nColour = RSN_GetLastProducedColour()' + sN + '				local oCharacteristics = oCard:GetCurrentCharacteristics()' + sN + '				if (nColour ~= COLOUR_COLOURLESS) then' + sN + '					oCharacteristics:GrantAbility(nColour)' + sN + '				else' + sN + '					oCharacteristics:GrantAbility(nDefaultColour)' + sN + '				end' + sN + '			end' + sN + '		</CONTINUOUS_ACTION>' + sN
		elif sDefaultColours and bComplex:
			sManaAbilities += '		<CONTINUOUS_ACTION layer="8">' + sN + '			local oCard = EffectSource()' + sN + '			if RSN_CheckSwitchToFallback(oCard) then' + sN + '				oCard:GetCurrentCharacteristics():GrantAbility(6)' + sN + '			end' + sN + '		</CONTINUOUS_ACTION>' + sN
		sManaAbilities += '	</STATIC_ABILITY>' + sN + '	<TRIGGERED_ABILITY forced_skip="1" replacement_effect="1">' + sN + '		<TRIGGER value="BEGINNING_OF_STEP" pre_trigger="1" />' + sN + '		<RESOLUTION_TIME_ACTION>' + sN + '			RSN_ClearProducedMana()' + sN + '		</RESOLUTION_TIME_ACTION>' + sN + '	</TRIGGERED_ABILITY>' + sN
		sManaAbilityBackupFormat = '	<MANA_ABILITY resource_id="sIndex">' + sN + '		<COST type="TapSelf" />' + sN + '		<PRODUCES amount="{sColour}" />' + sN + '	</MANA_ABILITY>' + sN
		sManaBackups = ''
		lstMana = ['W', 'U', 'B', 'R', 'G']
		for sMana in sDefaultColours.replace('-', ' ').split():
			if re.match(r"[-+]?\d+$", sMana) is not None:
				sManaBackups += sManaAbilityBackupFormat.replace('sIndex', '0').replace('sColour', sMana)
			elif lstMana.count(sMana):
				sManaBackups += sManaAbilityBackupFormat.replace('sIndex', str(lstMana.index(sMana)+1)).replace('sColour', sMana)
			else:
				sManaBackup = sManaAbilityBackupFormat.replace('sIndex', '6').replace('sColour', sMana)
				moManaMatch = re.search('<PRODUCES amount="{((?:[WUBRG]|[0-9]+)+)}" />', sManaBackup)
				if moManaMatch:
					sNewString = ''
					for sMana in moManaMatch.group(1):
						sNewString += '{' + sMana + '}'
					sManaBackup = sManaBackup.replace('<PRODUCES amount="{' + moManaMatch.group(1) + '}" />', '<PRODUCES amount="' + sNewString + '" />')
				sManaBackups += sManaBackup
		sManaAbilities += sManaBackups + sRegistrations
		sXml += sManaAbilities
		sXml = re.sub('\r\n<!--(COMPLEX|SIMPLE)MANAPRODUCTION-((?:(?:[WUBRGC]|[0-9]+)+-)+)->', '', sXml)
	sXml = re.sub('_Add\(PLUS', '_Add(', sXml)
	sXml = re.sub('_Add\(MINUS', '_Add(-', sXml)
	for sHelp in lstHelpNodes:
		sXml += sHelp
	return sXml

def ProcessAbility(crdCard, iIndex):
	sAbility = ""
	sHelp = ""
	# Just in case we need localized text, go ahead and extract it.
	sLocText = ""
	for lLang in MiscUtils.GetValues[Language]():
		sText = crdCard.GetAbilityText(iIndex, lLang)
		if len(sText) < 1:
			sText = crdCard.GetAbilityText(iIndex, lDefault)
		sLocText += LocalizedTextNode(lLang, sText).replace('  ', '	')
	# Abilities that contain the card's name need to have the cardname replaced so the dictionaries match.
	# search for simple templates
	sEnglishText = crdCard.GetAbilityText(iIndex, Language.English).Replace(crdCard.GetTitle(lDefault), "ENGLISHCARDNAME").Replace("{C}", "{1}")
	sEnglishText = re.sub(' <i>\((.*)</i>', '', sEnglishText)
	for sPattern, sTemplate in dicSimpleTemplates.iteritems():
		moMatch = re.search('^' + sPattern + '$', sEnglishText)
		if moMatch:
			if sPattern in dicSimpleTemplatesHelp.keys():
				sHelp = '	' + dicSimpleTemplatesHelp[sPattern] + sN
			if "<!--LOCALIZEDTEXT-->" in sTemplate:
				sTemplate = sTemplate.replace("<!--LOCALIZEDTEXT-->", sLocText)
			sTemplate = sTemplate.replace('\n', sN)
			sTemplate = "\n".join([ll.rstrip() for ll in sTemplate.splitlines() if ll.strip()])
			for iIndex in range(1, len(moMatch.groups())+1):
				sTemplate = sTemplate.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
			return '	' + sTemplate + sN, sHelp
	#It didn't find the ability as a whole, so now see if it is comma-delimited
	lstAbilities = sEnglishText.split(',') #Splits the ability into a list
	#There need to be at least 2. If there is only one, the first one should have found it. Since it didn't, we already know the ability isn't defined.
	if len(lstAbilities) > 1:
		iMatchedAbilities = 0
		#For each item in the list, strip outer spaces, make sure it actually exists, and then search the abilities for a match.
		for iIndexTemp, sAbility in enumerate(lstAbilities):
			sAbility = sAbility.strip()
			if len(sAbility) > 0:
				for sPattern, sTemplate in dicSimpleTemplates.iteritems():
					moMatch = re.search('^' + sPattern + '$', sAbility, re.IGNORECASE)
					if moMatch:
						iMatchedAbilities += 1
		#If it matched every ability, then define each one and concatenate them.
		if len(lstAbilities) == iMatchedAbilities:
			sAbilityRef = ""
			sHelpRef = ""
			lstAbilities = sEnglishText.split(',')
			if len(lstAbilities) > 1:
				for iIndexRef, sAbility in enumerate(lstAbilities):
					sAbility = sAbility.strip()
					if len(sAbility) > 0:
						for sPattern, sTemplate in dicSimpleTemplates.iteritems():
							moMatch = re.search('^' + sPattern + '$', sAbility, re.IGNORECASE)
							if moMatch:
								if sAbilityRef == "":
									sAbilityRef = '	' + sTemplate.replace('\n', sN)
								else:
									for sLetter, sReplacement in dicCapitalization.iteritems():
										for iIndexRef, sLanguageShort in enumerate(lstLanguageShort):
											sTemplate = sTemplate.replace(sLanguageShort + '"><![CDATA[' + sLetter, sLanguageShort + r""""><![CDATA[""" + sReplacement)
									sAbilityRef = sAbilityRef + sN + '	' + sTemplate.replace('\n', sN).replace('<STATIC_ABILITY>', r"""<STATIC_ABILITY commaspace="1">""")
								if sPattern in dicSimpleTemplatesHelp.keys():
									if sHelpRef == "":
										sHelpRef = '	' + dicSimpleTemplatesHelp[sPattern]
									else:
										sHelpRef = sHelpRef + sN + '	' + dicSimpleTemplatesHelp[sPattern]
				return sAbilityRef + sN, sHelpRef + sN
	# Try and match "Equipped creature" and "Enchanted creature" abilities.
	# At this point, we may need localized text, so extract it for use.
	sLocText = ""
	for lLang in MiscUtils.GetValues[Language]():
		sText = crdCard.GetAbilityText(iIndex, lLang)
		if len(sText) < 1:
			sText = crdCard.GetAbilityText(iIndex, lDefault)
		sLocText += LocalizedTextNode(lLang, sText).replace('  ', '	')
	sPower = crdCard.Power
	sToughness = crdCard.Toughness
	sPT = sPower + '/' + sToughness
	sEnglishTextTemp = sEnglishText.replace('first strike', 'firststrike').replace('double strike', 'doublestrike')
	moMatch = re.search('^(?:Equipped|Enchanted) creature has ([a-z]+|([a-z]+ and [a-z]+)|((([a-z]+, )+and [a-z]+))).$', sEnglishTextTemp)
	if moMatch:
		sAbility = """	<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					local oCharacteristics = oParent:GetCurrentCharacteristics()
					if oCharacteristics ~= nil then"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(1)):
				sAbilityTemp = lstPatternCharacteristics[Index]
				if sAbilityTemp == "firststrike":
					sAbilityTemp = "first_strike"
				elif sAbilityTemp == "doublestrike":
					sAbilityTemp = "double_strike"
				sAbility += sN + '						' + 'oCharacteristics:Bool_Set(CHARACTERISTIC_' + sAbilityTemp.upper() + ', 1)'
		sAbility += sN + '					end' + sN + '				end' + sN + '			end' + sN + '		</CONTINUOUS_ACTION>' + sN + '	</STATIC_ABILITY>'
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText).replace("<!--PT-->", sPT)
		return sAbility + sN, ""
	moMatch = re.search('^(?:Equipped|Enchanted) creature gets (\+|\-)([0-9])/(\+|\-)([0-9]) and has ([a-z]+|([a-z]+ and [a-z]+)|((([a-z]+, )+and [a-z]+))).$', sEnglishTextTemp)
	if moMatch:
		sAbility = """	<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					local oCharacteristics = oParent:GetCurrentCharacteristics()
					if oCharacteristics ~= nil then"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(5)):
				sAbilityTemp2 = lstPatternCharacteristics[Index]
				if sAbilityTemp2 == "firststrike":
					sAbilityTemp2 = "first_strike"
				elif sAbilityTemp2 == "doublestrike":
					sAbilityTemp2 = "double_strike"
				sAbility += sN + '						' + 'oCharacteristics:Bool_Set(CHARACTERISTIC_' + sAbilityTemp2.upper() + ', 1)'
		sAbility += """
					end
				end
			end
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="7C">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					local oCharacteristics = oParent:GetCurrentCharacteristics()
					if oCharacteristics ~= nil then
						oCharacteristics:Power_Add(#1#@2@)
						oCharacteristics:Toughness_Add(#3#@4@)
					end
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""
		for iIndex in range(1, len(moMatch.groups())+1):
			sAbility = sAbility.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText).replace("<!--PT-->", sPT)
		return sAbility + sN, ""
	moMatch = re.search('^Target creature has ([a-z]+|([a-z]+ and [a-z]+)|((([a-z]+, )+and [a-z]+))) until end of turn.$', sEnglishTextTemp)
	if moMatch:
		sAbility = """	<SPELL_ABILITY>
<!--LOCALIZEDTEXT-->		<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_GAIN"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(1)):
				sAbilityTemp = lstPatternCharacteristics[Index]
				if sAbilityTemp == "firststrike":
					sAbilityTemp = "first_strike"
				elif sAbilityTemp == "doublestrike":
					sAbilityTemp = "double_strike"
				sAbility += '_' + sAbilityTemp.upper()
		sAbility += """" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oController = EffectController()
			if oController ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_CONTROLLER, OP_IS, oController)
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
			end
		</TARGET_DEFINITION>
		<CONTINUOUS_ACTION layer="6">
			local oTarget = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				local oCharacteristics = oTarget:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(1)):
				sAbilityTemp = lstPatternCharacteristics[Index]
				if sAbilityTemp == "firststrike":
					sAbilityTemp = "first_strike"
				elif sAbilityTemp == "doublestrike":
					sAbilityTemp = "double_strike"
				sAbility += sN + '					' + 'oCharacteristics:Bool_Set(CHARACTERISTIC_' + sAbilityTemp.upper() + ', 1)'
		sAbility += sN + '			end' + sN + '		end' + sN + '		</CONTINUOUS_ACTION>' + sN + '	</SPELL_ABILITY>'
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText).replace("<!--PT-->", sPT)
		return sAbility + sN, ""
	moMatch = re.search('^Target creature gets (\+|\-)([0-9])/(\+|\-)([0-9]) and gains ([a-z]+|([a-z]+ and [a-z]+)|((([a-z]+, )+and [a-z]+))) until end of turn.$', sEnglishTextTemp)
	if moMatch:
		sAbility = """	<SPELL_ABILITY>
<!--LOCALIZEDTEXT-->		<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_GET_#1#_@2@_#3#_@4@"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(1)):
				sAbilityTemp = lstPatternCharacteristics[Index]
				if sAbilityTemp == "firststrike":
					sAbilityTemp = "first_strike"
				elif sAbilityTemp == "doublestrike":
					sAbilityTemp = "double_strike"
				sAbility += '_' + sAbilityTemp.upper()
		sAbility += """" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<CONTINUOUS_ACTION layer="6">
			local oTarget = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				local oCharacteristics = oTarget:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(5)):
				sAbilityTemp2 = lstPatternCharacteristics[Index]
				if sAbilityTemp2 == "firststrike":
					sAbilityTemp2 = "first_strike"
				elif sAbilityTemp2 == "doublestrike":
					sAbilityTemp2 = "double_strike"
				sAbility += sN + '					' + 'oCharacteristics:Bool_Set(CHARACTERISTIC_' + sAbilityTemp2.upper() + ', 1)'
		sAbility += """
				end
			end
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="7C">
			local oTarget = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				local oCharacteristics = oTarget:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Power_Add(#1#@2@)
					oCharacteristics:Toughness_Add(#3#@4@)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</SPELL_ABILITY>"""
		for iIndex in range(1, len(moMatch.groups())+1):
			sAbility = sAbility.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText).replace("<!--PT-->", sPT)
		return sAbility + sN, ""
	moMatch = re.search('^(?:\||)Rally(?:\||) — Whenever ENGLISHCARDNAME or another Ally enters the battlefield under your control, creatures you control gain ([a-z]+|([a-z]+ and [a-z]+)|((([a-z]+, )+and [a-z]+))) until end of turn.$', sEnglishTextTemp)
	if moMatch:
		sAbility = """	<TRIGGERED_ABILITY>
<!--LOCALIZEDTEXT-->		<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" />
		<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject():GetSubType():Test(CREATURE_TYPE_ALLY) and TriggerObject() ~= EffectSource()
		</TRIGGER>
		<FILTER filter_id="0">
			local oController = EffectController()
			if oController ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
				oFilter:Add(FE_CONTROLLER, OP_IS, oController)
			end
		</FILTER>
		<CONTINUOUS_ACTION layer="6" filter_id="0">
			local oFilteredCard = FilteredCard()
			if oFilteredCard ~= nil then
				local oCharacteristics = oFilteredCard:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then"""
		for Index in range(1,len(lstPatternCharacteristics)):
			if re.search(lstPatternCharacteristics[Index], moMatch.group(1)):
				sAbilityTemp2 = lstPatternCharacteristics[Index]
				if sAbilityTemp2 == "firststrike":
					sAbilityTemp2 = "first_strike"
				elif sAbilityTemp2 == "doublestrike":
					sAbilityTemp2 = "double_strike"
				sAbility += sN + '					' + 'oCharacteristics:Bool_Set(CHARACTERISTIC_' + sAbilityTemp2.upper() + ', 1)'
		sAbility += """
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</TRIGGERED_ABILITY>"""
		for iIndex in range(1, len(moMatch.groups())+1):
			sAbility = sAbility.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText).replace("<!--PT-->", sPT)
		return sAbility + sN, ""
	sAbility = ""
	sAbilityType = ""
	# search for header templates
	sHeader = ""
	for sPattern, sTemplate in dicPatternHeaders.iteritems():
		moMatch = re.search(sPattern, sEnglishText)
		if moMatch:
			if sPattern in dicPatternHeadersHelp.keys():
				sHelp = '	' + dicPatternHeadersHelp[sPattern] + sN
			# the ability type will be needed later
			sAbilityType = sTemplate[sTemplate.find('<')+1:sTemplate.find('_')]
			sHeader = '	' + sTemplate + sN
			break
	if len(sHeader) < 1:
		# try and guess ability type
		for sPattern, sTemplate in dicAbilityTypeTemplates.iteritems():
			moMatch = re.search(sPattern, sEnglishText)
			if moMatch:
				sAbilityType = sTemplate
				break
		if len(sAbilityType) < 1:
			if crdCard.HasType(CardType.Instant) or crdCard.HasType(CardType.Sorcery):
				sAbilityType = 'SPELL'
			else:
				sAbilityType = 'STATIC'
		sHeader = '	<' + sAbilityType + '_ABILITY>' + sN
	sAbility += sHeader
	sAbility += sLocText
	# in case of activated or utility ability, search for cost templates
	if sAbilityType in ['ACTIVATED', 'UTILITY']:
		sCost = ""
		for sPattern, sTemplate in dicPatternCosts.iteritems():
			moMatch = re.search(sPattern, sEnglishText)
			if moMatch:
				for iIndex in range(1, len(moMatch.groups())+1):
					sTemplate = sTemplate.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
				sCost += ('		' + sTemplate.replace('\n', sN) + sN)
		sAbility += sCost
	# in case of triggered ability, search for trigger templates
	elif sAbilityType == 'TRIGGERED':
		for sPattern, sTemplate in dicPatternTriggers.iteritems():
			moMatch = re.search(sPattern, sEnglishText.replace("Delirium — ", ""))
			if moMatch:
				for iIndex in range(1, len(moMatch.groups())+1):
					sTemplate = sTemplate.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
				sAbility += ('		' + sTemplate.replace('\n', sN) + sN)
				break
	# search for effect templates
	for sPattern, sTemplate in dicPatternEffects.iteritems():
		moMatch = re.search(sPattern, sEnglishText)
		if moMatch:
			if sPattern in dicPatternEffectsHelp.keys():
				sHelp = '	' + dicPatternEffectsHelp[sPattern] + sN
			for iIndex in range(1, len(moMatch.groups())+1):
				sTemplate = sTemplate.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
			sAbility += ('		' + sTemplate.replace('\n', sN) + sN)
			break
	#Didn't match any effect patterns. Last thing: tokens.
	if CommunityWad:
		moMatch = re.search('(?:: P|^P|, p)ut ' + NumberWordPattern() + ' ([0-9]+)/([[0-9]+) (white|blue|black|red|green|colorless)(?: and |)(white|blue|black|red|green|colorless)? (.*?) (creature|artifact creature|enchantment creature) token(?:s|)(?: with |)(.*) onto the battlefield(?:| (tapped)).(?:| (?:It has|They have) “Sacrifice this creature: Add \{1\} to your mana pool.”)$', sEnglishText)
		if moMatch:
			sColours = TrimSpecial(str(moMatch.group(4)) + ' ' + str(moMatch.group(5)))
			sSubTypes = str(moMatch.group(6)).replace(" ", "_").upper()
			sTypes = TrimSpecial(str(moMatch.group(7)))
			sAbilities = TrimSpecial(str(moMatch.group(8))) + "_"
			sTapped = AutoReplacePatternWords(str(moMatch.group(9)))
			sCount = str(iIndex)
			sToken = sSubTypes + "_" + SortToken(sTypes) + "_@2@_@3@_" + SortToken(sColours) + "_" + SortToken(sAbilities)
			sToken = sToken[:-1]
			if sToken == "ELDRAZI_SCION_C_@2@_@3@_C":
				sToken = "ELDRAZI_SCION_C_1_1_C_S_MM"
			sToken = sToken.replace("__", "_")
			if sTapped == "Tapped":
				sTapped = "_Tapped"
			else:
				sTapped = ""
			sFunction = '			CW_Tokens' + sTapped + '("' + sToken + '", #1#)'
			sAbility += '		<RESOLUTION_TIME_ACTION>' + sN + sFunction + sN + '		</RESOLUTION_TIME_ACTION>' + sN
			
			for iIndex in range(1, len(moMatch.groups())+1):	
				sAbility = sAbility.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
		else:
			moMatch = re.search('(?:: C|^C|, c)reate ' + NumberWordPattern() + ' ([0-9]+)/([[0-9]+) (white|blue|black|red|green|colorless)(?: and |)(white|blue|black|red|green|colorless)? (.*?) (creature|artifact creature|enchantment creature) token(?:s|)(?: with |)(.*)$', sEnglishText)
			if moMatch:
				sColours = TrimSpecial(str(moMatch.group(4)) + ' ' + str(moMatch.group(5)))
				sSubTypes = str(moMatch.group(6)).replace(" ", "_").upper()
				sTypes = TrimSpecial(str(moMatch.group(7)))
				sAbilities = TrimSpecial(str(moMatch.group(8))) + "_"
				sCount = str(iIndex)
				sToken = sSubTypes + "_" + SortToken(sTypes) + "_@2@_@3@_" + SortToken(sColours) + "_" + SortToken(sAbilities)
				sToken = sToken[:-1]
				sToken = sToken.replace("__", "_")
				sFunction = '			CW_Tokens("' + sToken + '", #1#)'
				sAbility += '		<RESOLUTION_TIME_ACTION>' + sN + sFunction + sN + '		</RESOLUTION_TIME_ACTION>' + sN
				
				for iIndex in range(1, len(moMatch.groups())+1):	
					sAbility = sAbility.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
	if "<!--LOCALIZEDTEXT-->" in sAbility:
		sAbility = sAbility.replace("<!--LOCALIZEDTEXT-->", sLocText)
	if "<!--PT-->" in sAbility:
		sAbility = sAbility.replace("<!--PT-->", sPT)
	sAbility += NodeEnd(sAbility)
	return ' ' + sAbility, sHelp

def SimpleNode(sNodeName, sAttributeName, sAttributeValue):
	return '	<' + sNodeName + ' ' + sAttributeName + '="' + sAttributeValue + '" />' + sN

def LocalizedTextNode(lLanguage, sContents):
	return '		<LOCALISED_TEXT LanguageCode="' + Constants.LanguageToCode(lLanguage) + '"><![CDATA[' + sContents + ']]></LOCALISED_TEXT>' + sN

def NodeEnd(sNode):
	moMatch = re.search(r'^(\s*)<(\w+)(?=[> ])', sNode)
	if moMatch:
		return moMatch.group(1) + '</' + moMatch.group(2) + '>' + sN
	else:
		return ""

def SimpleCostPattern(NoCapture = False):
	sPattern = r'('
	if NoCapture:
		sPattern += '?:'
	sPattern += '(?:\{(?:\d+|['
	for ceElem in MiscUtils.GetValues[CostElement]():
		if not ceElem in [CostElement.Tap, CostElement.Untap, CostElement.Planeswalk, CostElement.Chaos]:
			sPattern += re.escape(Constants.CostElementToTextString(ceElem))
	sPattern += r'])\})+)'
	return sPattern

def SortToken(sInput):
	lstAlphabet = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '_']
	sOutput = ''
	for sLetter in lstAlphabet:
		if sLetter in sInput:
			sOutput += sLetter
	if sOutput == '_':
		sOutput = ''
	return sOutput

def TrimSpecial(sInput):
	sOutput = ''
	for i in sInput.replace('first strike', 'F').replace('double strike', 'D').replace('blue', 'U').replace(' and ', ' ').replace(',', '').upper().replace(' NONE', '').split():
		sOutput += i[0]
	return sOutput

def NumberWordPattern(NoCapture = False):
	sPattern = '('
	if NoCapture:
		sPattern += '?:'
	for i in range(1,21):
		sPattern += NumberToWord(i) + '|'
	sPattern = sPattern[:-1] + ')'
	return sPattern
		
def NumberToWord(iInt):
	if iInt == 1:
		return '(?:a|an)'
	if iInt == 2:
		return 'two'
	if iInt == 3:
		return 'three'
	if iInt == 4:
		return 'four'
	if iInt == 5:
		return 'five'
	if iInt == 6:
		return 'six'
	if iInt == 7:
		return 'seven'
	if iInt == 8:
		return 'eight'
	if iInt == 9:
		return 'nine'
	if iInt == 10:
		return 'ten'
	if iInt == 11:
		return 'eleven'
	if iInt == 12:
		return 'twelve'
	if iInt == 13:
		return 'thirteen'
	if iInt == 14:
		return 'fourteen'
	if iInt == 15:
		return 'fifteen'
	if iInt == 16:
		return 'sixteen'
	if iInt == 17:
		return 'seventeen'
	if iInt == 18:
		return 'eighteen'
	if iInt == 19:
		return 'nineteen'
	if iInt == 20:
		return 'twenty'

def SimpleCounterPattern(NoCapture = False):
	sPattern = '('
	if NoCapture:
		sPattern += '?:'
	for sCounter, sRef in dicPatternSimpleCounters.iteritems():
		sPattern += sCounter + '|'
	sPattern = sPattern[:-1] + ')'
	return sPattern
	
def AutoReplacePatternWords(sInput):
	sInput = str(sInput)
	if sInput in dicAutoReplacePatternWords:
		return dicAutoReplacePatternWords[sInput]
	elif sInput == 'None':
		return ''
	else:
		return sInput.title().replace('{', '').replace('}', '')

def SimpleCharacteristicsPattern(NoCapture = False):
	sPattern = '('
	if NoCapture:
		sPattern += '?:'
	for sCharacteristic in lstPatternCharacteristics:
		sPattern += sCharacteristic + '|'
	sPattern = sPattern[:-1] + ')'
	return sPattern

def Test():
	crdDummy = Card()
	crdDummy.SetTitle(Language.English, StringUtils.GracefulQuotes('Some Card'))
	crdDummy.MultiverseID = '9009'
	crdDummy.CastingCost = '{W}'
	crdDummy.AddType('Creature')
	crdDummy.AddSubType('Soldier')
	crdDummy.Power = '1'
	crdDummy.Toughness = '3'
	crdDummy.Expansion = 'TST'
	crdDummy.SetRarity('C')
	crdDummy.Artist = 'TestArtist'
	for iIndex, sAbility in enumerate(lstTestAbilities):
		crdDummy.SetAbilityText(iIndex, Language.English, StringUtils.GracefulQuotes(sAbility))
	print Get(crdDummy, '').replace(sN, '\n')

	# Dictionaries/Lists - The following are dictionaries and lists used for various lookups.
dicPatternSimpleCounters = dict([
	('\+1/\+1', 'MTG():PlusOnePlusOneCounters()'),
	('\+1/\+0', 'MTG():PlusOnePlusZeroCounters()'),
	('\+0/\+1', 'MTG():PlusZeroPlusOneCounters()'),
	('\-1/\-1', 'MTG():MinusOneMinusOneCounters()')
])

lstPatternCharacteristics = [
	'deathtouch',
	'defender',
	'doublestrike',
	'firststrike',
	'flash',
	'flying',
	'haste',
	'hexproof',
	'indestructible',
	'intimidate',
	'plainswalk',
	'islandwalk',
	'swampwalk',
	'mountainwalk',
	'forestwalk',
	'lifelink',
	'reach',
	'shroud',
	'trample',
	'vigilance',
	'flanking',
	'phasing',
	'changeling',
	'fear',
	'infect',
	'wither'	
]

dicPatternNumberWords = dict([
	('a', '1'),
	('an', '1'),
	('two', '2'),
	('three', '3'),
	('four', '4'),
	('five', '5'),
	('six', '6'),
	('seven', '7'),
	('eight', '8'),
	('nine', '9'),
	('ten', '10'),
	('X', 'GetPaidX()')
])

dicManaShortLongCodes = dict([
	('a', 'B/G'),
	('b', 'B/R'),
	('c', 'G/U'),
	('d', 'G/W'),
	('e', 'R/G'),
	('f', 'R/W'),
	('g', 'U/B'),
	('h', 'U/R'),
	('i', 'W/B'),
	('j', 'W/U'),
	('H', 'W/P'),
	('I', 'R/P'),
	('J', 'B/P'),
	('K', 'U/P'),
	('L', 'G/P')
])

dicAutoReplacePatternWords = dict([
	('a', '1'),
	('an', '1'),
	('two', '2'),
	('three', '3'),
	('four', '4'),
	('five', '5'),
	('six', '6'),
	('seven', '7'),
	('eight', '8'),
	('nine', '9'),
	('ten', '10'),
	('eleven', '11'),
	('twelve', '12'),
	('thirteen', '13'),
	('fourteen', '14'),
	('fifteen', '15'),
	('sixteen', '16'),
	('seventeen', '17'),
	('eighteen', '18'),
	('nineteen', '19'),
	('twenty', '20'),
	('+1/+1', 'MTG():PlusOnePlusOneCounters()'),
	('+1/+0', 'MTG():PlusOnePlusZeroCounters()'),
	('+0/+1', 'MTG():PlusZeroPlusOneCounters()'),
	('-1/-1', 'MTG():MinusOneMinusOneCounters()'),
	('+', 'PLUS'),
	('-', 'MINUS')
])

#--------------------------------------------------------------------------------------------
# SIMPLE TEMPLATES
# The following templates are placed in the card when the exact ability name has been found.
#--------------------------------------------------------------------------------------------
dicSimpleTemplates = dict([

	("First strike",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[First strike]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Initiative]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Daña primero.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Erstschlag]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Attacco improvviso]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[先制攻撃]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[선제공격]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Первый удар]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Iniciativa]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FIRST_STRIKE" />
	</STATIC_ABILITY>"""),

	("Trample",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Trample]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Piétinement]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Arrolla.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Verursacht Trampelschaden]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Travolgere]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[トランプル]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[돌진]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Пробивной удар]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Atropelar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_TRAMPLE" />
	</STATIC_ABILITY>"""),

	("Double strike",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Double strike]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Double initiative]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Daña dos veces.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Doppelschlag]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Doppio attacco]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[二段攻撃]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[이단공격]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Двойной удар]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Golpe duplo]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_DOUBLE_STRIKE" />
	</STATIC_ABILITY>"""),

	("Deathtouch",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Deathtouch]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Contact mortel]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Toque mortal.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Todesberührung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Tocco letale]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[接死]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[치명타]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Смертельное касание]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Toque mortífero]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_DEATHTOUCH" />
	</STATIC_ABILITY>"""),

	("Lifelink",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Lifelink]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Lien de vie]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Vínculo vital.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Lebensverknüpfung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Legame vitale]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[絆魂]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[생명연결]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Цепь жизни]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Vínculo com a vida]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_LIFELINK" />
	</STATIC_ABILITY>"""),

	("Reach",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Reach]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Portée]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Alcance.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Reichweite]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Raggiungere]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[到達]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[대공]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Захват]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Alcance]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_REACH" />
	</STATIC_ABILITY>"""),

	("Defender",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Defender]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Défenseur]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Defensor.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Verteidiger]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Difensore]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[防衛]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[수비태세]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защитник]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Defensor]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_DEFENDER" />
	</STATIC_ABILITY>"""),

	("Shroud",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Shroud]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Linceul]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Velo.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Verhüllt]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Velo]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[被覆]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[은폐]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Пелена]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Manto]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_SHROUD" />
	</STATIC_ABILITY>"""),

	("Indestructible",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Indestructible]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Indestructible]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Indestructible.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Unzerstörbar]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Indistruttibile]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[破壊不能]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[무적]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Неразрушимость]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Indestrutível]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_INDESTRUCTIBLE" />
	</STATIC_ABILITY>"""),

	("Flying",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Flying]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Vol]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Vuela.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Fliegend]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Volare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[飛行]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[비행]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Полет]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Voar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FLYING" />
	</STATIC_ABILITY>"""),

	("Shadow",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Shadow]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Distorsion]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Se desvanece.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Irrealität]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Ombra]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[シャドー]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Shadow]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Тень]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Sombra]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="6">
			RSN_Characteristics_Set(EffectSource(), CHARACTERISTIC_SHADOW, 1)
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_LIBRARY">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP and MTG():GetTurnNumber() == 0
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			RSN_Characteristics_CreateManagers()
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TOKEN_REGISTRATION reservation="1" type="_MANAGER_CHARACTERISTICS" />"""),

	("Flash",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Flash]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Flash]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Destello.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Aufblitzen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Lampo]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[瞬速]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[섬광]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Миг]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Lampejo]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FLASH" />
	</STATIC_ABILITY>"""),

	("Vigilance",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Vigilance]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Vigilance]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Vigilancia.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Wachsamkeit]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Cautela]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[警戒]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[경계]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Бдительность]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Vigilância]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_VIGILANCE" />
	</STATIC_ABILITY>"""),

	("Haste",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Haste]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Célérité]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Prisa.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Eile]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Rapidità]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[速攻]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[신속]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Ускорение]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Ímpeto]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_HASTE" />
	</STATIC_ABILITY>"""),

	("Fear",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Fear]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Peur]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Inspirar temor.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Verursacht Furcht]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Paura]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[畏怖]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[공포]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Страх]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Amedrontar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FEAR" />
	</STATIC_ABILITY>"""),

	("Plainswalk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Plainswalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Traversée des plaines]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cruza llanuras.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Ebenentarnung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Passa-pianure]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[平地渡り]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Plainswalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Знание Равнин]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Travessia de planície]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_PLAINSWALK" />
	</STATIC_ABILITY>"""),

	("Islandwalk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Islandwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Traversée des îles]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cruza islas.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Inseltarnung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Passa-Isole]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[島渡り]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Islandwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Знание Островов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Travessia de ilha]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_ISLANDWALK" />
	</STATIC_ABILITY>"""),

	("Swampwalk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Swampwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Traversée des marais]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cruza pantanos]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Sumpftarnung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Passa-Paludi]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[沼渡り]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[늪잠입]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Знание Болот]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Travessia de pântano]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_SWAMPWALK" />
	</STATIC_ABILITY>"""),

	("Mountainwalk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Mountainwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Traversée des montagnes]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cruza montañas.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Gebirgstarnung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Passa-montagne]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[山渡り]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Mountainwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Знание Гор]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Travessia de Montanha]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_MOUNTAINWALK" />
	</STATIC_ABILITY>"""),

	("Forestwalk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Forestwalk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Traversée des forêts]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cruza bosques.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Waldtarnung]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Passa-Foreste]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[森渡り]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[숲잠입]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Знание Лесов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Travessia de floresta]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FORESTWALK" />
	</STATIC_ABILITY>"""),

	("Hexproof",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Hexproof]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Défense talismanique]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Antimaleficio.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Fluchsicher]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Anti-malocchio]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[呪禁]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[방호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Порчеустойчивость]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Resistência à magia]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_HEXPROOF" />
	</STATIC_ABILITY>"""),

	("Intimidate",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Intimidate]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Intimidation]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Intimidar.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Einschüchtern]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Intimidire]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[威嚇]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[위협]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Устрашение]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Intimidar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_INTIMIDATE" />
	</STATIC_ABILITY>"""),

	("Infect",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Infect]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Infettare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Infizieren]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Infection]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Infectar.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[感染]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[감염]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Инфекция]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Infectar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_INFECT" />
	</STATIC_ABILITY>"""),

	("Wither",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Wither]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Flétrissure]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Debilitar.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Verdorren]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Avvizzire]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[萎縮]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[고사]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Увядание]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Murchar]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_WITHER" />
	</STATIC_ABILITY>"""),

	("Battle cry",
r"""<TRIGGERED_ABILITY badge="BADGE_BATTLE_CRY">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Battle cry]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Cri de guerre]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Grito de batalla.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schlachtruf]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Grido di guerra]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[喊声]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[전투함성]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Боевой клич]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Grito de guerra]]></LOCALISED_TEXT>
		<TRIGGER value="ATTACKING" simple_qualifier="self" />
		<FILTER filter_id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_IS_ATTACKING, true)
			oFilter:Add(FE_CARD_INSTANCE, OP_NOT, TriggerObject())
		</FILTER>
		<CONTINUOUS_ACTION layer="7C" filter_id="0">
			local oFilteredCard = FilteredCard()
			if oFilteredCard ~= nil then
				local oCharacteristics = oFilteredCard:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Power_Add(1)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</TRIGGERED_ABILITY>"""),

	("Retrace",
r"""<UTILITY_ABILITY qualifier="Retrace" active_zone="ZONE_GRAVEYARD">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Rievocare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Retrace]]></LOCALISED_TEXT>
		<COST type="Discard" definition="0" compartment="2" query_tag="CARD_QUERY_CHOOSE_A_LAND_TO_DISCARD" item_count="1" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add( FE_TYPE, OP_IS, CARD_TYPE_LAND )
		</COST_DEFINITION>
	</UTILITY_ABILITY>"""),

	("ENGLISHCARDNAME can't attack alone.",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<INTRINSIC characteristic="CHARACTERISTIC_CANNOT_ATTACK_ALONE" />
	</STATIC_ABILITY>"""),

	("ENGLISHCARDNAME can't block alone.",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<INTRINSIC characteristic="CHARACTERISTIC_CANNOT_BLOCK_ALONE" />
	</STATIC_ABILITY>"""),

	("ENGLISHCARDNAME can't attack or block alone.$",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<INTRINSIC characteristic="CHARACTERISTIC_CANNOT_ATTACK_ALONE" />
		<INTRINSIC characteristic="CHARACTERISTIC_CANNOT_BLOCK_ALONE" />
	</STATIC_ABILITY>"""),

	("Exalted",
r"""<TRIGGERED_ABILITY badge="BADGE_EXALTED">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Exalted]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Exaltation]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Exaltado.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Edelmut]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Esaltato]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[賛美]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[고귀]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Повышение]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Exaltado]]></LOCALISED_TEXT>
		<TRIGGER value="ATTACKING_ALONE" simple_qualifier="objectyoucontrol" />
		<CONTINUOUS_ACTION layer="7C">
			local oTriggerObject = TriggerObject()
			if oTriggerObject ~= nil then
				local oCharacteristics = oTriggerObject:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Power_Add(1)
					oCharacteristics:Toughness_Add(1)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
		<AUTO_SKIP no_trigger_object="1" />
	</TRIGGERED_ABILITY>"""),

	("Undying",
r"""<TRIGGERED_ABILITY badge="BADGE_UNDYING">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Undying]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Survivance]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Resiliencia.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Unverwüstlich]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Immortale]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[不死]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[불사]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Нетленность]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Imortal]]></LOCALISED_TEXT>
		<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="self" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD" />
		<INTERVENING_IF ignore_resolution_check="1">
			return TriggerObject() ~= nil and TriggerObject():CountCounters(MTG():PlusOnePlusOneCounters()) == 0
		</INTERVENING_IF>
		<RESOLUTION_TIME_ACTION>
			local oTriggerObject = TriggerObject()
			if oTriggerObject ~= nil then
				oTriggerObject:PutOntoBattlefield(TriggerObject():GetOwner())
				oTriggerObject:AddCounters(MTG():PlusOnePlusOneCounters(), 1)
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),

	("Changeling",
r"""<STATIC_ABILITY active_zone="ZONE_ANY">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Changeling]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Changelin]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Cambiaformas.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Wandelwicht]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Cangiante]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[多相]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[자유변형]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Перевертыш]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Morfoloide]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="4">
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:GetCurrentCharacteristics():Bool_Set(CHARACTERISTIC_CHANGELING, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Persist",
r"""<TRIGGERED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Persist]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Persistance]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Persistir.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Beharrlichkeit]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Persistere]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[頑強]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[집요]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Упорство]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Persistir]]></LOCALISED_TEXT>
		<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="self" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD" />
		<INTERVENING_IF ignore_resolution_check="1">
			return TriggerObject() ~= nil and TriggerObject():CountCounters(MTG():MinusOneMinusOneCounters()) == 0
		</INTERVENING_IF>
		<RESOLUTION_TIME_ACTION>
			local oTriggerObject = TriggerObject()
			if oTriggerObject ~= nil then
				oTriggerObject:PutOntoBattlefield(oTriggerObject:GetOwner())
				oTriggerObject:AddCounters(MTG():MinusOneMinusOneCounters(), 1)
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),

	("Flanking",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Flanking]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Débordement]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Flanquea.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Flankenangriff]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Aggirare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[側面攻撃]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[협공]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Обход]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Flanquear]]></LOCALISED_TEXT>
		<INTRINSIC characteristic="CHARACTERISTIC_FLANKING" />
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY badge="BADGE_FLANKING">
		<TRIGGER value="WAS_BLOCKED_BY_A_CREATURE" simple_qualifier="self">
			return SecondaryObject() ~= nil and SecondaryObject():GetCurrentCharacteristics():Bool_Get(CHARACTERISTIC_FLANKING) == false
		</TRIGGER>
		<CONTINUOUS_ACTION layer="7C">
			local oBlocker = SecondaryObject()
			if oBlocker ~= nil then
				local oCharacteristics = oBlocker:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Power_Add(-1)
					oCharacteristics:Toughness_Add(-1)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</TRIGGERED_ABILITY>"""),

	("Totem armor",
r"""<TRIGGERED_ABILITY replacement_query="1">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Totem armor]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Armure totémique]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Armadura tótem.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Totembeistand]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Armatura totem]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[族霊鎧]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[토템아머]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Тотемная броня]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Armadura de totem]]></LOCALISED_TEXT>
		<TRIGGER value="DESTROYED" pre_trigger="1">
			local oTriggerObject = TriggerObject()
			local oSource = EffectSource()
			if oTriggerObject ~= nil and oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil and oTriggerObject == oParent then
					local oFilter = ClearFilter()
					oFilter:Add(FE_PARENT_CARD, OP_IS, oTriggerObject)
					oFilter:Add(FE_CHARACTERISTIC, OP_HAS, CHARACTERISTIC_TOTEM_ARMOUR)
					oFilter:EvaluateObjects()
					if oSource == oFilter:GetNthEvaluatedObject(0) then
						MTG():OverrideEvent()
						return true
					end
				end
			end
			return false
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			local oTriggerObject = TriggerObject()
			if oSource ~= nil and oTriggerObject ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					oParent:ClearDamage()
				end
				local oFilter = ClearFilter()
				oFilter:Add(FE_PARENT_CARD, OP_IS, oTriggerObject)
				oFilter:Add(FE_CHARACTERISTIC, OP_HAS, CHARACTERISTIC_TOTEM_ARMOUR)
				local iCount = oFilter:EvaluateObjects()
				if iCount &gt; 0 then
					if iCount == 1 then
						oSource:Destroy()
					else
						oTriggerObject:GetPlayer():ChooseItem("CARD_QUERY_CHOOSE_ENCHANTMENT_TO_DESTROY", EffectDC():Make_Targets(0))
					end
				end
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<STATIC_ABILITY>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:GetCurrentCharacteristics():Bool_Set(CHARACTERISTIC_TOTEM_ARMOUR, 1)
			end
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="8">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					oParent:GetCurrentCharacteristics():Badge_Set(BADGE_TOTEM_ARMOR)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	
	("Prowess",
r"""<TRIGGERED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Prowess]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Prouesse]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Destreza.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Bravour]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Prodezza]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[果敢]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[기량]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Искусность]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Destreza]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[灵技]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[靈技]]></LOCALISED_TEXT>
		<TRIGGER value="SPELL_PLAYED" simple_qualifier="controller">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE) == false
		</TRIGGER>
		<CONTINUOUS_ACTION layer="7C">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Power_Add(1)
					oCharacteristics:Toughness_Add(1)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</TRIGGERED_ABILITY>"""),
	
	("Rebound",
r"""<SPELL_ABILITY active_zone="ZONE_ANY">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Rebound]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Rebond]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Rebote.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Abprall]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Ripresa]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[反復]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[반동]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Отскок]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Rebote]]></LOCALISED_TEXT>
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil and oSource:GetErstwhileZone() == ZONE_HAND then
				local oDelayDC = EffectDC():Make_Chest(3)
				oDelayDC:Set_CardPtr(0, oSource)
				MTG():CreateDelayedTrigger(2, oDelayDC)
				oSource:Exile()
			end
		</RESOLUTION_TIME_ACTION>
	</SPELL_ABILITY>
	<TRIGGERED_ABILITY resource_id="2">
		<TRIGGER value="BEGINNING_OF_PLAYERS_STEP" simple_qualifier="controller">
			return MTG():GetStep() == STEP_UPKEEP
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			local oSource = EffectSource()
			if oController ~= nil and oSource ~= nil then
				oController:BeginNewMultipleChoice()
					oController:AddMultipleChoiceAnswer("CARD_QUERY_CASCADE_CAST_SPELL", oController:CanCastSpellForFree(oSource))
					oController:AddMultipleChoiceAnswer("CARD_QUERY_CASCADE_DO_NOT_CAST")
				oController:AskMultipleChoiceQuestion("CARD_QUERY_MC_CAST_THIS_SPELL", oSource)
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			local oController = EffectController()
			if oSource ~= nil and oController ~= nil and oController:GetMultipleChoiceResult() == 0 and oController:CanCastSpellForFree(oSource) then
				oController:CastSpellForFree(oSource)
			end
		</RESOLUTION_TIME_ACTION>
		<CLEANUP fire_once="1" />
	</TRIGGERED_ABILITY>"""),
	
	("Exploit",
r"""<TRIGGERED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Exploit]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Exploitation]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Aprovechar.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Ausschlachten]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Sfruttare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[濫用]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[수탈]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Угнетение]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Explorar]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[榨取]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[搾取]]></LOCALISED_TEXT>
		<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" />
		<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			if oController ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
				oFilter:Add(FE_CONTROLLER, OP_IS, oController)
				oController:ChooseItem("CARD_QUERY_CHOOSE_CREATURE_TO_SACRIFICE", EffectDC():Make_Targets(0), QUERY_FLAG_MAY)
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			local oCreature = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			local oController = EffectController()
			if oCreature ~= nil and oController ~= nil then
				EffectDC():Get_Targets(0):LKIShield_CardPtr(0)
				oController:Sacrifice(oCreature)
				EffectDC():Set_Int(1, 1)
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			if EffectDC():Get_Int(1) == 1 then
				<!--Exploited a creature code here, include int check in any additional actions-->
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),
	
	("Dash " + SimpleCostPattern(),
r"""<UTILITY_ABILITY qualifier="Alternate">
<!--LOCALIZEDTEXT-->
		<COST type="Generic">
			<PREREQUISITE>
				local oController = EffectController()
				if oController ~= nil then
					local oFilter = ClearFilter()
					oFilter:Add(FE_CARD_NAME, OP_IS, "WARBRINGER")
					oFilter:Add(FE_CONTROLLER, OP_IS, oController)
					oFilter:Add(FE_LUA_CONDITION, 1, oController, EffectDC())
					return (oFilter:CountStopAt(1) == 1 and oController:CanPayManaCost("<!--ReducedCost-->")) or oController:CanPayManaCost("@1@")
				end
				return false
			</PREREQUISITE>
			<PLAY_TIME_ACTION>
				local oController = EffectController()
				if oController ~= nil then
					local oFilter = ClearFilter()
					oFilter:Add(FE_CARD_NAME, OP_IS, "WARBRINGER")
					oFilter:Add(FE_CONTROLLER, OP_IS, oController)
					oFilter:Add(FE_LUA_CONDITION, 1, oController, EffectDC())
					if oFilter:CountStopAt(1) == 1 and oController:CanPayManaCost("<!--ReducedCost-->") then
						oController:PayManaCost("<!--ReducedCost-->")
					elseif oController:CanPayManaCost("@1@") then
						oController:PayManaCost("@1@")
					end
				end
			</PLAY_TIME_ACTION>
		</COST>
		<FILTER_CONDITION id="1">
			return RSN_Characteristics_Get(FilteredCard(), NEO_ABILITIES_ACTIVE)
		</FILTER_CONDITION>
		<COST type="Generic">
			<PREREQUISITE>
				return true
			</PREREQUISITE>
			<RESOLUTION_TIME_ACTION>
				local oSource = EffectSource()
				if oSource ~= nil then
					local oDelayDC = EffectDC():Make_Chest(21)
					oDelayDC:Set_CardPtr(20, oSource)
					oDelayDC:Protect_CardPtr(20)
					MTG():CreateDelayedTrigger(21, oDelayDC)
				end
			</RESOLUTION_TIME_ACTION>
		</COST>
		<ABILITY_TEXT tag="CW_SLU_ALTERNATE_COST_PAY_DASH" />
	</UTILITY_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" resource_id="21">
		<CLEANUP simple_cleanup="EndOfTurn" fire_once="1" />
		<TRIGGER value="SPELL_RESOLVED">
			return TriggerObject() ~= nil and TriggerObject() == EffectDC():Get_CardPtr(20)
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			MTG():CreateDelayedTrigger(22, nil)
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TRIGGERED_ABILITY resource_id="22" priority="20" linked_ability_group="1" replacement_effect="1">
		<CLEANUP fire_once="1" />
		<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_STACK" />
		<RESOLUTION_TIME_ACTION>
			LinkedDC():Set_CardPtr(21, EffectSource())
		</RESOLUTION_TIME_ACTION>
		<AUTO_SKIP no_effect_source="1" />
	</TRIGGERED_ABILITY>
	<TRIGGERED_ABILITY priority="-1" linked_ability_group="1">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_END_OF_TURN and EffectSource() ~= nil and LinkedDC():Get_CardPtr(21) == EffectSource()
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:PutInHand()
			end
		</RESOLUTION_TIME_ACTION>
		<AUTO_SKIP no_effect_source="1" />
	</TRIGGERED_ABILITY>
	<STATIC_ABILITY linked_ability_group="1">
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil and LinkedDC():Get_CardPtr(21) == oSource then
				oSource:GetCurrentCharacteristics():Bool_Set(CHARACTERISTIC_HASTE, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	
	("Menace",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Menace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Menace]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Amenaza.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Bedrohlich]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Minacciare]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[威迫]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[호전적]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Угроза]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Ameaçar]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[威慑]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[威懾]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				RSN_Characteristics_Set(oSource, CW_CHARACTERISTIC_MENACE, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_LIBRARY">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP and MTG():GetTurnNumber() == 0
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			RSN_Characteristics_CreateManagers()
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TOKEN_REGISTRATION reservation="1" type="_MANAGER_CHARACTERISTICS" />"""),
	
	("Skulk",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Skulk]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Furtivité]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Escurridizo.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schleichen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Furtivo]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[潜伏]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[은폐]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Скрытность]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Esquivo]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[潜匿]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[潛匿]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				RSN_Characteristics_Set(oSource, CW_CHARACTERISTIC_SKULK, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_LIBRARY">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP and MTG():GetTurnNumber() == 0
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			RSN_Characteristics_CreateManagers()
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TOKEN_REGISTRATION reservation="1" type="_MANAGER_CHARACTERISTICS" />"""),
	
	("Protection from all colors",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from all colors]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre toutes les couleurs]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra todos los colores.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor allen Farben]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione da tutti i colori]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（すべての色）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[모든 색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от всех цветов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra todas as cores]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_COLOUR, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_COLOUR, OP_IS, COLOUR_BLUE)
				oSubfilter:Add(FE_COLOUR, OP_IS, COLOUR_GREEN)
				oSubfilter:Add(FE_COLOUR, OP_IS, COLOUR_RED)
				oSubfilter:Add(FE_COLOUR, OP_IS, COLOUR_WHITE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	 
	 
	("Protection from artifacts",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from artifacts]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les artefacts]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra artefactos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Artefakten]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dagli artefatti]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（アーティファクト）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от артефактов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra artefatos]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ARTIFACT)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	 
	("Protection from Beasts",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Beasts]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les bêtes]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Bestias.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Bestien]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dalle Bestie]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Bestas]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_BEAST)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from black",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from black]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le noir]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra negro.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Schwarz]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal nero]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（黒）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[흑색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от черного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o preto]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反黑保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反黑保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_COLOUR, OP_IS, COLOUR_BLACK)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	 
	("Protection from black and from green",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from black and from green]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le noir, protection contre le vert]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra negro y contra verde.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Schwarz und vor Grün]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal nero, protezione dal verde]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（黒）、プロテクション（緑）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[흑색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от черного, Защита от зеленого]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o preto, proteção contra o verde]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_GREEN)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	 
	("Protection from black and from red",
r"""<STATIC_ABILITY">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from black and from red]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le noir, protection contre le rouge]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra negro y contra rojo.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Schwarz und vor Rot]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal nero, protezione dal rosso]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（黒）、プロテクション（赤）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[흑색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от черного, Защита от красного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o preto, proteção contra o vermelho]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_RED)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from blue",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from blue]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le bleu]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra azul]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Blau]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal blu]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（青）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[청색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от синего]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o azul]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反蓝保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反藍保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_COLOUR, OP_IS, COLOUR_BLUE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from blue and from black",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from blue and from black]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le bleu, protection contre le noir]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra azul y contra negro.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Blau und vor Schwarz]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal blu, protezione dal nero]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（青）、プロテクション（黒）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[청색으로부터 보호, 흑색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от синего, Защита от черного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o azul, proteção contra o preto]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLUE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from blue and from red",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from blue and from red]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le bleu, protection contre le rouge.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Blau, Schutz vor Rot]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLUE)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_RED)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from blue, from black, and from red",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from blue, from black, and from red]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le bleu, contre le noir et contre le rouge]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra azul, contra negro y contra rojo.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Blau, vor Schwarz und vor Rot]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal blu, dal nero e dal rosso.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（青）、プロテクション（黒）、プロテクション（赤）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Protection from blue, from black, and from red]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от синего, от черного и от красного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o azul, o preto e o vermelho.]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLUE)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_RED)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from clerics",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Clerics]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les clercs]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Clérigos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Klerikern]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dai Chierici]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Clérigos]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_CLERIC)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from converted mana cost 3 or greater",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from converted mana cost 3 or greater]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra coste de maná convertido de 3 o mayor.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[保护（总法术力大于或等于3）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от конвертированной мана-стоимости 3 или больше]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra custo de mana convertido maior ou igual a 3.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（点数で見たマナ・コストが３以上）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione da costo di mana convertito pari o superiore a 3.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le coût converti de mana 3 ou plus]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor umgewandelten Manakosten von 3 oder höher]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_CMC, OP_GREATER_THAN_OR_EQUAL_TO, 3)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from creatures",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from creatures]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les créatures]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra criaturas.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Kreaturen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dalle creature]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra criaturas]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from demons and from dragons",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Demons and from Dragons]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les démons et protection contre les dragons]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Demonios y contra Dragones.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Dämonen und vor Drachen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione da Demoni e da Draghi]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（デーモン）、プロテクション（ドラゴン）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[악마 및 용으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Демонов и от Драконов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Demônios e contra Dragões]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反恶魔保护，反龙保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反惡魔保護，反龍保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_DEMON)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_DRAGON)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from dragons",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Dragons]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les dragons]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Dragones.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Drachen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dai Draghi]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Dragões]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_DRAGON)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Elves",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Elves]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les elfes]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra elfos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Elfen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dagli Elfi]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（エルフ）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Эльфов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Elfos]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反地精保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反妖精保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_ELF)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from enchantments",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from enchantments]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les enchantements]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra encantamientos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Verzauberungen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dagli incantesimi]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（エンチャント）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от чар]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra encantamentos]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反结界保护]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ENCHANTMENT)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Goblins",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Goblins]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les gobelins]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Trasgos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Goblins]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dai Goblin]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（ゴブリン）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Гоблинов]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Goblins]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反精灵保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反鬼怪保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_GOBLIN)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from green",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from green]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le vert]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra verde.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Grün]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal verde]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（緑）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от зеленого]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o verde]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反绿保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反綠保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_COLOUR, OP_IS, COLOUR_GREEN)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from instants",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from instants]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les éphémères]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra instantáneos.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Spontanzaubern]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dagli istantanei]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（インスタント）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[순간마법으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от мгновенных заклинаний]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra mágicas instantâneas]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_INSTANT)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Kavu",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Kavu]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les kavrus]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Kavu]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_KAVU)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from lands",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from lands]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les terrains]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra tierras.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Ländern]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dalle terre]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（土地）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от земель]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra terrenos]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反地保护]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_LAND)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from legendary creatures",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from legendary creatures]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les légendes.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Legenden]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_And()
				oSubfilter:Add(FE_SUPERTYPE, OP_IS, SUPERTYPE_LEGENDARY)
				oSubfilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from non-Spirit creatures",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from non-Spirit creatures]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les créatures non-esprit]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra criaturas que no sean Espíritus.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Nicht-Geist-Kreaturen]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dalle creature non-Spirito]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra criaturas que não sejam Espíritos]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反非精怪生物保护]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_And()
				oSubfilter:Add(FE_SUBTYPE, OP_NOT, CREATURE_TYPE_SPIRIT)
				oSubfilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from red",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from red]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le rouge]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra rojo.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Rot]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal rosso]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（赤）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[적색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от красного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o vermelho]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反红保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反紅保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_COLOUR, OP_IS, COLOUR_RED)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from red and from green",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from red and from green]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le rouge et protection contre le vert]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra rojo y contra verde.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Rot, Schutz vor Grün]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal rosso e protezione dal verde]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（緑）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от красного и от зеленого]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o vermelho e contra o verde]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_GREEN)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_RED)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from red and from white",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from red and from white]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le rouge et protection contre le blanc]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra rojo y contra blanco.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Rot und vor Weiß]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal rosso e dal bianco]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（赤）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Protection from red and from white]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от красного и от белого]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o vermelho e contra o branco]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_RED)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_WHITE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from snow",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from snow]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le neigeux]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra nevado.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Verschneitem]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dalla neve]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（氷雪）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от снега]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra neve]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUPERTYPE, OP_IS, SUPERTYPE_SNOW)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Vampires",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Vampires]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les vampires]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra vampiros.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Vampiren]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dai Vampiri]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（吸血鬼）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[흡혈귀로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Вампиров]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Vampiros]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反吸血鬼保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反吸血鬼保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_VAMPIRE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Vampires, from Werewolves, and from Zombies",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Vampires, from Werewolves, and from Zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les vampires, contre les loups-garous et contre les zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Vampiros, Licántropos y Zombies.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Vampiren, Werwölfen und Zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dai Vampiri, dai Mannari e dagli Zombie]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（吸血鬼）、プロテクション（狼男）、プロテクション（ゾンビ）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[흡혈귀로부터 보호, 늑대인간으로부터 보호, 좀비로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Вампиров, от Вервольфов и от Зомби]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Vampiros, Lobisomens e Zumbis.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反吸血鬼保护， 反狼人保护， 反灵俑保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反吸血鬼保護， 反狼人保護， 反殭屍保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_VAMPIRE)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_WEREWOLF)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_ZOMBIE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from white",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from white]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le blanc]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra blanco.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Weiß]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal bianco]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（白）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[백색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от белого]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o branco]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反白保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反白保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_COLOUR, OP_IS, COLOUR_WHITE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from white and from black",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from white and from black]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le blanc, protection contre le noir]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra blanco y contra negro.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Weiß und vor Schwarz]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal bianco, protezione dal nero]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（白）、プロテクション（黒）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[백색으로부터 보호, 흑색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от белого, Защита от черного]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o branco, Proteção contra o preto]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLACK)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_WHITE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from white and from blue",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from white and from blue]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre le blanc, protection contre le bleu]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra blanco y contra azul.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Weiß und vor Blau]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dal bianco, protezione dal blu]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（白）、プロテクション（青）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[백색으로부터 보호, 청색으로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от белого, Защита от синего]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra o branco, Proteção contra o azul]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				local oSubfilter = oFilter:AddSubFilter_Or()
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_BLUE)
				oSubfilter:Add(FE_SUBTYPE, OP_IS, COLOUR_WHITE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("Protection from Zombies",
r"""<STATIC_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Protection from Zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Protection contre les zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Protección contra Zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Schutz vor Zombies]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Protezione dagli Zombie]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[プロテクション（ゾンビ）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[좀비로부터 보호]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Защита от Зомби]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Proteção contra Zumbis]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[反灵俑保护]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[反殭屍保護]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="0">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oFilter = ClearFilter()
				oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_ZOMBIE)
				oSource:Protection()
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("ENGLISHCARDNAME enters the battlefield tapped.",
r"""<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_TRANSITION">
<!--LOCALIZEDTEXT-->		<TRIGGER value="ZONECHANGE_TRANSITION" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY" pre_trigger="1" />
		<RESOLUTION_TIME_ACTION>
			local oCard = TriggerObject()
			if oCard ~= nil then
				oCard:Tap()
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),

	("ENGLISHCARDNAME doesn’t untap during your untap step.",
r"""<TRIGGERED_ABILITY replacement_effect="1" forced_skip="1">
<!--LOCALIZEDTEXT-->		<TRIGGER value="BEGINNING_OF_PLAYERS_STEP">
			return MTG():GetStep() == STEP_CLEANUP
		</TRIGGER>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Bool_Set(CHARACTERISTIC_DOESNT_UNTAP, 1)
				end
			end
		</CONTINUOUS_ACTION>
		<DURATION>
			return RSN_UntilEndOfUntapForPlayerOnly(EffectController())
		</DURATION>
	</TRIGGERED_ABILITY>"""),

	("ENGLISHCARDNAME can’t be (?:countered.|countered by spells or abilities.)",
r"""<STATIC_ABILITY active_zone="ZONE_STACK">
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="8">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Bool_Set(CHARACTERISTIC_CANT_BE_COUNTERED, 1)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("ENGLISHCARDNAME can’t block.",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Bool_Set(CHARACTERISTIC_CANT_BLOCK, 1)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("ENGLISHCARDNAME can’t be blocked.",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="8">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Bool_Set(CHARACTERISTIC_UNBLOCKABLE, 1)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	
	("Outlast " + SimpleCostPattern() ,
r"""<ACTIVATED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Outlast @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Résilience @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Supervivencia @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Standhaft @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Perdurare @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[長久@1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[버티기 @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Стойкость @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Sobreviver @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[延生@1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[延生@1@]]></LOCALISED_TEXT>
		<AVAILABILITY sorcery_time="1" />
		<COST mana_cost="@1@" type="Mana" />
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(MTG():PlusOnePlusOneCounters(), 1)
			end
		</RESOLUTION_TIME_ACTION>
	</ACTIVATED_ABILITY>"""),

	("ENGLISHCARDNAME enters the battlefield with " + NumberWordPattern() + " " + SimpleCounterPattern() + " counter(?:|s) on it.",
r"""<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_TRANSITION">
<!--LOCALIZEDTEXT-->		<TRIGGER value="ZONECHANGE_TRANSITION" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY" />
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(#2#, #1#)
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),

	("ENGLISHCARDNAME enters the battlefield with " + NumberWordPattern() + " ([a-z]+) counter(?:|s) on it.",
r"""<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_TRANSITION">
<!--LOCALIZEDTEXT-->		<TRIGGER value="ZONECHANGE_TRANSITION" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY" />
		<!--COUNTERREGISTRATION-->		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(#2#, #1#)
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),

	("ENGLISHCARDNAME attacks each turn if able.",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="8">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oCharacteristics = oSource:GetCurrentCharacteristics()
				if oCharacteristics ~= nil then
					oCharacteristics:Bool_Set(CHARACTERISTIC_MUST_ATTACK_EACH_TURN, 1)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),

	("As an additional cost to cast ENGLISHCARDNAME, sacrifice a creature.",
r"""<UTILITY_ABILITY qualifier="Additional">
<!--LOCALIZEDTEXT-->		<COST type="Sacrifice" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CREATURE_TO_SACRIFICE" item_count="1" LKI_shield="1" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</COST_DEFINITION>
	</UTILITY_ABILITY>"""),
	
	("Renown ([1-9])",
r"""<TRIGGERED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Renown @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Réputation @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Prestigio @1@.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Ruhm @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Rinomare @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[高名@1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[명성 @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Прославленность @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Renome @1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[铭勇@1@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[銘勇@1@]]></LOCALISED_TEXT>
		<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" simple_qualifier="self" damage_type="combat" />
		<INTERVENING_IF>
			return RSN_ObjectDC():Get_Int(CW_RENOWNED) == 0
		</INTERVENING_IF>
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(MTG():PlusOnePlusOneCounters(), @1@)
				RSN_ObjectDC():Set_Int(CW_RENOWNED, 1)
				RSN_ObjectDC():Set_Int(CW_TRIGGER, CW_TRIGGER_BECAME_RENOWNED)
				MTG():FireTrigger(TRIGGER_PHENOMENON_ENCOUNTERED)
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),
	
	("(?:Equipped|Enchanted) creature gets (\+|\-)([0-9])/(\+|\-)([0-9]).",
r"""<STATIC_ABILITY>
<!--LOCALIZEDTEXT-->		<CONTINUOUS_ACTION layer="7C">
			local oSource = EffectSource()
			if oSource ~= nil then
				local oParent = oSource:GetParent()
				if oParent ~= nil then
					oParent:GetCurrentCharacteristics():Power_Add(#1#@2@)
					oParent:GetCurrentCharacteristics():Toughness_Add(#3#@4@)
				end
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	
	("Target creature gets \-([0-9])/\-([0-9]) until end of turn.",
r"""<SPELL_ABILITY>
<!--LOCALIZEDTEXT-->
		<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_LOSE_@1@_@2@" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add( FE_TYPE, OP_IS, CARD_TYPE_CREATURE )
		</TARGET_DEFINITION>
		<CONTINUOUS_ACTION layer="7C">
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				local oCharacteristics = oTarget:GetCurrentCharacteristics()
				oCharacteristics:Power_Add(-@1@)
				oCharacteristics:Toughness_Add(-@2@)
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
		<AI_SIMPLIFIED_TARGETING compartment="0" hint="HINT_ENEMY_ONLY" />
	</SPELL_ABILITY>"""),
	
	("Target creature gets (\+|\-)([0-9])/(\+|\-)([0-9]) until end of turn.",
r"""<SPELL_ABILITY>
<!--LOCALIZEDTEXT-->
		<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_GET_#1#_@2@_#3#_@4@" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add( FE_TYPE, OP_IS, CARD_TYPE_CREATURE )
		</TARGET_DEFINITION>
		<CONTINUOUS_ACTION layer="7C">
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				local oCharacteristics = oTarget:GetCurrentCharacteristics()
				oCharacteristics:Power_Add(#1#@2@)
				oCharacteristics:Toughness_Add(#3#@4@)
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
	</SPELL_ABILITY>"""),
	
	("Awaken ([0-9]+)—" + SimpleCostPattern(),
r"""<ACTIVATED_ABILITY forced_skip="1" qualifier="Alternate" active_zone="ZONE_HAND" linked_ability_group="1">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Awaken @1@—@2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Éveil @1@ — @2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Despertar @1@—@2@.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Erwecken @1@ — @2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Risveglio @1@—@2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[覚醒@1@―@2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[각성 @1@—@2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Пробуждение @1@ — @2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Despertar @1@ — @2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[醒转@1@～@2@]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[醒轉@1@～@2@]]></LOCALISED_TEXT>
		<COST type="Generic">
			<PREREQUISITE>
				return CW_Awaken_Activated_PreReq()
			</PREREQUISITE>
			<PLAY_TIME_ACTION>
				RSN_MarkManaAbilityStart()
			</PLAY_TIME_ACTION>
			<PLAY_TIME_ACTION>
				CW_Awaken_Activated_PTA(21) --Utility cost resource_id
			</PLAY_TIME_ACTION>
		</COST>
		<RESOLUTION_TIME_ACTION>
			EffectController():CastSpellUsingResourceCost(EffectSource(), 21)
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
	</ACTIVATED_ABILITY>
	<STATIC_ABILITY linked_ability_group="1" active_zone="ZONE_ANY">
		<CONTINUOUS_ACTION layer="0">
			CW_Awaken_Any_CA0(20) --Awaken resource_id
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="6">
			CW_Awaken_Any_CA6()
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<STATIC_ABILITY linked_ability_group="1" active_zone="ZONE_EXILE">
		<CONTINUOUS_ACTION layer="0">
			CW_Awaken_Any_CA0(20) --Awaken resource_id
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="6">
			CW_Awaken_Any_CA6()
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<STATIC_ABILITY linked_ability_group="1" active_zone="ZONE_HAND">
		<CONTINUOUS_ACTION layer="1">
			CW_Awaken_Hand_CA1(21) --Utility cost resource_id
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<SPELL_ABILITY resource_id="20" linked_ability_group="1"> --Awaken effect
		<PLAY_TIME_ACTION>
			CW_Awaken_PTA()
		</PLAY_TIME_ACTION>
		<TARGET tag="CARD_QUERY_CHOOSE_LAND_TO_ANIMATE" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			CW_Awaken_Target()
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			CW_Awaken_RTA(@1@) --Number of counters
		</RESOLUTION_TIME_ACTION>
		<CONTINUOUS_ACTION layer="4">
			CW_Awaken_CA4()
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="6">
			CW_Awaken_CA6()
		</CONTINUOUS_ACTION>
		<CONTINUOUS_ACTION layer="7B">
			CW_Awaken_CA7B()
		</CONTINUOUS_ACTION>
		<DURATION>
			return CW_Awaken_Duration()
		</DURATION>
	</SPELL_ABILITY>
	<TRIGGERED_ABILITY linked_ability_group="1" replacement_effect="1" active_zone="ZONE_ANY">
		<TRIGGER value="SPELL_RESOLVED">
			return TriggerObject() == Object()
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			CW_Awaken_SpellResolved_RTA()
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<UTILITY_ABILITY resource_id="21">
		<COST mana_cost="@2@" type="Mana" />
	</UTILITY_ABILITY>"""),
	
	("Devoid",
r"""<STATIC_ABILITY active_zone="ZONE_ANY">
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Devoid]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Carence]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Vacío.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Fahl]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Vacuità]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[欠色]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[결여]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Лишение]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Desprovido]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[虚色]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[虛色]]></LOCALISED_TEXT>
		<CONTINUOUS_ACTION layer="5">
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:GetCurrentCharacteristics():Colour_Set(COLOUR_COLOURLESS)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>"""),
	
	("Ingest",
r"""<TRIGGERED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Ingestion]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Ingerir.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Zehren]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Ingerire]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Ingerir]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[Ingest]]></LOCALISED_TEXT>
		<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" simple_qualifier="self" damage_type="combat" />
		<RESOLUTION_TIME_ACTION>
			local oPlayer = SecondaryPlayer()
			if oPlayer ~= nil then
				local oCard = oPlayer:Library_GetTop()
				if oCard ~= nil then
					oCard:Exile()
				end
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),
	
	("Surge " + SimpleCostPattern(),
r"""<UTILITY_ABILITY qualifier="Alternate">
<!--LOCALIZEDTEXT-->
		<COST mana_cost="@1@" type="Mana" />
		<COST type="Generic">
			<PREREQUISITE>
				return CW_General_SurgeAvailability()
			</PREREQUISITE>
			<PLAY_TIME_ACTION>
				CW_General_SurgePaid()
			</PLAY_TIME_ACTION>
		</COST>
		<ABILITY_TEXT tag="CW_ALTERNATE_COST_PAY_SURGE" />
	</UTILITY_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_ANY">
		<TRIGGER value="ZONECHANGE_CONSIDERED" pre_trigger="1" simple_qualifier="self" to_zone="ZONE_STACK" from_zone="ZONE_ANY" />
		<TRIGGER value="ZONECHANGE_CONSIDERED" pre_trigger="1" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_STACK" />
		<RESOLUTION_TIME_ACTION>
			CW_General_SurgeProtectDC()
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>"""),
	
	("Emerge " + SimpleCostPattern(),
r"""<UTILITY_ABILITY qualifier="Alternate">
<!--LOCALIZEDTEXT-->
		<COST type="Generic">
			<PREREQUISITE>
				return CW_Abilities_EmergePreReq()
			</PREREQUISITE>
			<PLAY_TIME_ACTION>
				CW_Abilities_EmergePTA1()
			</PLAY_TIME_ACTION>
			<PLAY_TIME_ACTION>
				CW_Abilities_EmergePTA2(!!!COLOURLESS_COMPONENT!!!, "{!!!COLORED_COMPONENT!!!}")
			</PLAY_TIME_ACTION>
		</COST>
		<FILTER_CONDITION id="1">
			return CW_Abilities_EmergeFilterCondition(6, "{B}")
		</FILTER_CONDITION>
		<ABILITY_TEXT tag="CW_ALTERNATE_COST_PAY_EMERGE" />
	</UTILITY_ABILITY>"""),
	
	("Fabricate ([0-9]+)",
r"""<TRIGGERED_ABILITY>
<!--LOCALIZEDTEXT-->
		<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" />
		<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			local oSource = EffectSource()
			if oController ~= nil and oSource ~= nil then
				oController:BeginNewMultipleChoice()
					oController:AddMultipleChoiceAnswer("CARD_QUERY_FABRICATE_CHOOSE_COUNTERS")
					oController:AddMultipleChoiceAnswer("CARD_QUERY_FABRICATE_CHOOSE_TOKENS")
				oController:AskMultipleChoiceQuestion("CARD_QUERY_FABRICATE", oSource)
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			local oSource = EffectSource()
			if oController ~= nil then
				if oSource == nil or oController:GetMultipleChoiceResult() == 1 then
					CW_Tokens("SERVO_AC_1_1_C", @1@)
				else
					oSource:AddCounters(MTG():PlusOnePlusOneCounters(), @1@)
				end
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		
		("Crew ([0-9]+)",
r"""<ACTIVATED_ABILITY>
		<LOCALISED_TEXT LanguageCode="en-US"><![CDATA[Crew 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="fr-FR"><![CDATA[Pilotage 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="es-ES"><![CDATA[Tripular 1 |(<!--PT-->)|.]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="de-DE"><![CDATA[Bemannen 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="it-IT"><![CDATA[Manovrare 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="jp-JA"><![CDATA[搭乗１（<!--PT-->）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ko-KR"><![CDATA[탑승 1 (<!--PT-->)]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="ru-RU"><![CDATA[Экипаж 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="pt-BR"><![CDATA[Tripular 1 |(<!--PT-->)|]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-CN"><![CDATA[搭载1（<!--PT-->）]]></LOCALISED_TEXT>
		<LOCALISED_TEXT LanguageCode="zh-HK"><![CDATA[搭載1（<!--PT-->）]]></LOCALISED_TEXT>
		<COST type="Generic">
			<PREREQUISITE>
				return CW_Abilities_CrewPreReq(3) -- Crew number
			</PREREQUISITE>
			<RESOLUTION_TIME_ACTION repeating="1">
				return CW_Abilities_CrewRTA1(3) -- Crew number
			</RESOLUTION_TIME_ACTION>
			<RESOLUTION_TIME_ACTION>
				CW_Abilities_CrewRTA2()
			</RESOLUTION_TIME_ACTION>
		</COST>
		<FILTER_CONDITION id="1">
			return CW_Abilities_CrewFilterCondition()
		</FILTER_CONDITION>
		<CONTINUOUS_ACTION layer="4">
			CW_Abilities_CrewCA4()
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />
		<AUTO_SKIP>
			return CW_Abilities_CrewAutoSkip()
		</AUTO_SKIP>
	</ACTIVATED_ABILITY>"""),
	
	("Embalm " + SimpleCostPattern(),
r"""<ACTIVATED_ABILITY active_zone="ZONE_GRAVEYARD">
<!--LOCALIZEDTEXT-->
		<COST mana_cost="@1@" type="Mana" />
		<COST type="ExileSelf" />
		<AVAILABILITY sorcery_time="1" />
		<RESOLUTION_TIME_ACTION>
			CW_Tokens("EMBALM_CARDNAMETAG")
			CW_General_FireTrigger(CW_TRIGGER_EMBALM, EffectSource())
		</RESOLUTION_TIME_ACTION>
	</ACTIVATED_ABILITY>
	<STATIC_ABILITY>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				RSN_Characteristics_Set(EffectSource(), CW_CHARACTERISTIC_EMBALM, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_ANY">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP and MTG():GetTurnNumber() == 0
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			CW_General_CreateManagers("_MANAGER_CHARACTERISTICS")
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TOKEN_REGISTRATION reservation="1" type="_MANAGER_CHARACTERISTICS" />"""),
	
	("Eternalize " + SimpleCostPattern(),
r"""<ACTIVATED_ABILITY active_zone="ZONE_GRAVEYARD">
<!--LOCALIZEDTEXT-->
		<COST mana_cost="@1@" type="Mana" />
		<COST type="ExileSelf" />
		<AVAILABILITY sorcery_time="1" />
		<RESOLUTION_TIME_ACTION>
			CW_Tokens("ETERNALIZE_CARDNAMETAG")
			CW_General_FireTrigger(CW_TRIGGER_EMBALM, EffectSource())
		</RESOLUTION_TIME_ACTION>
	</ACTIVATED_ABILITY>
	<STATIC_ABILITY>
		<CONTINUOUS_ACTION layer="6">
			local oSource = EffectSource()
			if oSource ~= nil then
				RSN_Characteristics_Set(EffectSource(), CW_CHARACTERISTIC_ETERNALIZE, 1)
			end
		</CONTINUOUS_ACTION>
	</STATIC_ABILITY>
	<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_ANY">
		<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP and MTG():GetTurnNumber() == 0
		</TRIGGER>
		<RESOLUTION_TIME_ACTION>
			CW_General_CreateManagers("_MANAGER_CHARACTERISTICS")
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>
	<TOKEN_REGISTRATION reservation="1" type="_MANAGER_CHARACTERISTICS" />"""),
	
	("Afflict  ([0-9]+)",
r"""<TRIGGERED_ABILITY>
<!--LOCALIZEDTEXT-->
		<TRIGGER value="WAS_BLOCKED" simple_qualifier="self"
		<RESOLUTION_TIME_ACTION>
			local oDefendingPlayer = EffectSource() and EffectSource():GetPlayerAttacked()
			if oDefendingPlayer ~= nil then
				oDefendingPlayer:LoseLife(1)
			end
		</RESOLUTION_TIME_ACTION>
	</ACTIVATED_ABILITY>"""),
	
	("\|\(Loyalty ([0-9]+)\)\|",
r"""<TRIGGERED_ABILITY replacement_effect="1" active_zone="ZONE_TRANSITION">
<!--LOCALIZEDTEXT-->
		<COUNTER_REGISTRATION name="Loyalty" proliferate="11" />
		<TRIGGER value="ZONECHANGE_TRANSITION" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY" />
		<RESOLUTION_TIME_ACTION>
			if EffectSource() ~= nil then
				EffectSource():AddCounters( MTG():GetCountersType("Loyalty"), @1@ )
			end
		</RESOLUTION_TIME_ACTION>
	</TRIGGERED_ABILITY>""")
])
	
	
dicSimpleTemplatesHelp = dict([
	("First strike", '<HELP title="MORE_INFO_BADGE_TITLE_0" body="MORE_INFO_BADGE_BODY_0" zone="ZONE_ANY" />'),
	("Trample", '<HELP title="MORE_INFO_BADGE_TITLE_1" body="MORE_INFO_BADGE_BODY_1" zone="ZONE_ANY" />'),
	("Double strike", '<HELP title="MORE_INFO_BADGE_TITLE_2" body="MORE_INFO_BADGE_BODY_2" zone="ZONE_ANY" />'),
	("Deathtouch", '<HELP title="MORE_INFO_BADGE_TITLE_3" body="MORE_INFO_BADGE_BODY_3" zone="ZONE_ANY" />'),
	("Lifelink", '<HELP title="MORE_INFO_BADGE_TITLE_4" body="MORE_INFO_BADGE_BODY_4" zone="ZONE_ANY" />'),
	("Reach", '<HELP title="MORE_INFO_BADGE_TITLE_5" body="MORE_INFO_BADGE_BODY_5" zone="ZONE_ANY" />'),
	("Defender", '<HELP title="MORE_INFO_BADGE_TITLE_6" body="MORE_INFO_BADGE_BODY_6" zone="ZONE_ANY" />'),
	("Shroud", '<HELP title="MORE_INFO_BADGE_TITLE_7" body="MORE_INFO_BADGE_BODY_7" zone="ZONE_ANY" />'),
	("Indestructible", '<HELP title="MORE_INFO_BADGE_TITLE_8" body="MORE_INFO_BADGE_BODY_8" zone="ZONE_ANY" />'),
	("Flying", '<HELP title="MORE_INFO_BADGE_TITLE_10" body="MORE_INFO_BADGE_BODY_10" zone="ZONE_ANY" />'),
	("Shadow", '<HELP title="MORE_INFO_BADGE_TITLE_11" body="MORE_INFO_BADGE_BODY_11" zone="ZONE_ANY" />'),
	("Flash", '<HELP title="MORE_INFO_BADGE_TITLE_12" body="MORE_INFO_BADGE_BODY_12" zone="ZONE_ANY" />'),
	("Vigilance", '<HELP title="MORE_INFO_BADGE_TITLE_13" body="MORE_INFO_BADGE_BODY_13" zone="ZONE_ANY" />'),
	("Haste", '<HELP title="MORE_INFO_BADGE_TITLE_14" body="MORE_INFO_BADGE_BODY_14" zone="ZONE_ANY" />'),
	("Fear", '<HELP title="MORE_INFO_BADGE_TITLE_15" body="MORE_INFO_BADGE_BODY_15" zone="ZONE_ANY" />'),
	("Plainswalk", '<HELP title="MORE_INFO_BADGE_TITLE_16" body="MORE_INFO_BADGE_BODY_16" zone="ZONE_ANY" />'),
	("Islandwalk", '<HELP title="MORE_INFO_BADGE_TITLE_17" body="MORE_INFO_BADGE_BODY_17" zone="ZONE_ANY" />'),
	("Swampwalk", '<HELP title="MORE_INFO_BADGE_TITLE_18" body="MORE_INFO_BADGE_BODY_18" zone="ZONE_ANY" />'),
	("Mountainwalk", '<HELP title="MORE_INFO_BADGE_TITLE_19" body="MORE_INFO_BADGE_BODY_19" zone="ZONE_ANY" />'),
	("Forestwalk", '<HELP title="MORE_INFO_BADGE_TITLE_20" body="MORE_INFO_BADGE_BODY_20" zone="ZONE_ANY" />'),
	("Hexproof", '<HELP title="MORE_INFO_BADGE_TITLE_21" body="MORE_INFO_BADGE_BODY_21" zone="ZONE_ANY" />'),
	("Intimidate", '<HELP title="MORE_INFO_BADGE_TITLE_24" body="MORE_INFO_BADGE_BODY_24" zone="ZONE_ANY" />'),
	("Battle cry", '<HELP title="MORE_INFO_BADGE_TITLE_26" body="MORE_INFO_BADGE_BODY_26" zone="ZONE_ANY" />'),
	("Infect", '<HELP title="MORE_INFO_BADGE_TITLE_31" body="MORE_INFO_BADGE_BODY_31" zone="ZONE_ANY" />'),
	("Wither", '<HELP title="MORE_INFO_BADGE_TITLE_32" body="MORE_INFO_BADGE_BODY_32" zone="ZONE_ANY" />'),
	("Exalted", '<HELP title="MORE_INFO_BADGE_TITLE_33" body="MORE_INFO_BADGE_BODY_33" zone="ZONE_ANY" />'),
	("Undying", '<HELP title="MORE_INFO_BADGE_TITLE_34" body="MORE_INFO_BADGE_BODY_34" zone="ZONE_ANY" />'),
	("Changeling", '<HELP title="MORE_INFO_BADGE_TITLE_35" body="MORE_INFO_BADGE_BODY_35" zone="ZONE_ANY" />'),
	("Persist", '<HELP title="MORE_INFO_TITLE_PERSIST" body="MORE_INFO_BODY_PERSIST" zone="ZONE_ANY" />'),
	("Flanking", '<HELP title="MORE_INFO_BADGE_TITLE_30" body="MORE_INFO_BADGE_BODY_30" zone="ZONE_ANY" />'),
	("Totem armor", '<HELP title="MORE_INFO_BADGE_TITLE_28" body="MORE_INFO_BADGE_BODY_28" zone="ZONE_ANY" />'),
	("Prowess", '<HELP title="CW_MORE_INFO_BADGE_TITLE_PROWESS" body="CW_MORE_INFO_BADGE_BODY_PROWESS" zone="ZONE_ANY" />'),
	("Rebound", '<HELP title="CW_MORE_INFO_TITLE_REBOUND" body="CW_MORE_INFO_BODY_REBOUND" zone="ZONE_ANY" />'),
	("Exploit", '<HELP title="CW_MORE_INFO_TITLE_EXPLOIT" body="CW_MORE_INFO_BODY_EXPLOIT" zone="ZONE_ANY" />'),
	("Dash " + SimpleCostPattern(), '<HELP title="CW_MORE_INFO_TITLE_DASH" body="CW_MORE_INFO_BODY_DASH" zone="ZONE_ANY" />'),
	("Menace", '<HELP title="CW_MORE_INFO_TITLE_MENACE" body="CW_MORE_INFO_BODY_MENACE" zone="ZONE_ANY" />'),
	("Bolster", '<HELP title="CW_MORE_INFO_TITLE_BOLSTER" body="CW_MORE_INFO_BODY_BOLSTER" zone="ZONE_ANY" />'),
	("Renown ([1-9])", '<HELP title="CW_MORE_INFO_TITLE_RENOWN" body="CW_MORE_INFO_BODY_RENOWN" zone="ZONE_ANY" />'),
	("Outlast " + SimpleCostPattern(), '<HELP title="CW_MORE_INFO_TITLE_OUTLAST" body="CW_MORE_INFO_BODY_OUTLAST" zone="ZONE_ANY" />'),
	("Surge " + SimpleCostPattern(), '<HELP title="CW_MORE_INFO_TITLE_SURGE" body="CW_MORE_INFO_BODY_SURGE" zone="ZONE_ANY" />'),
	("Skulk", '<HELP title="CW_MORE_INFO_TITLE_SKULK" body="CW_MORE_INFO_BODY_SKULK" zone="ZONE_ANY" />'),
	("Protection from all colors", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from artifacts", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Beasts", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from black", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from black and from green", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from black and from red", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from blue", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from blue and from black", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from blue and from red", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from blue, from black, and from red", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from clerics", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from converted mana cost 3 or greater", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from creatures", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from demons and from dragons", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from dragons", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Elves", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from enchantments", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Goblins", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from green", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from instants", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Kavu", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from lands", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from legendary creatures", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from non-Spirit creatures", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from red", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from red and from green", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from red and from white", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from snow", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Vampires", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Vampires, from Werewolves, and from Zombies", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from white", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from white and from black", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from white and from blue", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Protection from Zombies", '<HELP title="MORE_INFO_TITLE_PROTECTION" body="MORE_INFO_BODY_PROTECTION" zone="ZONE_ANY" />'),
	("Awaken ([0-9]+)—" + SimpleCostPattern(), '<HELP title="MORE_INFO_TITLE_AWAKEN" body="MORE_INFO_BODY_AWAKEN" zone="ZONE_ANY" />'),
	("Devoid", '<HELP title="MORE_INFO_TITLE_DEVOID" body="MORE_INFO_BODY_DEVOID" zone="ZONE_ANY" />'),
	("Ingest", '<HELP title="MORE_INFO_TITLE_INGEST" body="MORE_INFO_BODY_INGEST" zone="ZONE_ANY" />'),
	("Fabricate ([0-9]+)", '<HELP title="MORE_INFO_BADGE_TITLE_FABRICATE" body="MORE_INFO_BADGE_BODY_FABRICATE" zone="ZONE_ANY" />'),
	("Crew ([0-9]+)", '<HELP title="MORE_INFO_BADGE_TITLE_CREW" body="MORE_INFO_BADGE_BODY_CREW" zone="ZONE_ANY" />'),
	("Embalm " + SimpleCostPattern(), '<HELP title="MORE_INFO_BADGE_TITLE_EMBALM" body="MORE_INFO_BADGE_BODY_EMBALM" zone="ZONE_ANY" />'),
	("Eternalize " + SimpleCostPattern(), '<HELP title="MORE_INFO_BADGE_TITLE_ETERNALIZE" body="MORE_INFO_BADGE_BODY_ETERNALIZE" zone="ZONE_ANY" />'),
	("Afflict ([0-9]+)", '<HELP title="MORE_INFO_BADGE_TITLE_AFFLICT" body="MORE_INFO_BADGE_BODY_AFFLICT" zone="ZONE_ANY" />')
])

#--------------------------------------------------------------------------------------------
# ABILITY-TYPE-GUESSING TEMPLATES
# The following templates are used in order to guess what kind of ability needs to be added.
#--------------------------------------------------------------------------------------------
dicAbilityTypeTemplates = dict([
	(r'^([^“]+: |Equip ' + SimpleCostPattern() + ')', 'ACTIVATED'),
	(r'^(?:|(?:|\|)Delirium|Landfall|Imprint(?:|\|) — )(When |Whenever |If |At |ENGLISHCARDNAME enters )', 'TRIGGERED'),
	(r'(^As an additional cost )', 'UTILITY')
])

#--------------------------------------------------------------------------------------------
# PATTERN TEMPLATES
# The following templates are placed in the card when a certain regex pattern has been found.
# In the template, @n@ will be substituted by the n-th group in the regex.
#--------------------------------------------------------------------------------------------

dicPatternHeaders = dict([
	(r'^Enchant creature$', '<SPELL_ABILITY attach_definition="0">'),
	(r'^Enchant creature\W', '<SPELL_ABILITY attach_definition="0">'),
	(r'^Flashback\W', '<UTILITY_ABILITY qualifier="Flashback" active_zone="ZONE_GRAVEYARD">'),
	(r'^Replicate\W', '<UTILITY_ABILITY qualifier="Replicate">')
])

dicPatternHeadersHelp = dict([
	(r'^Flashback\W', '<HELP title="MORE_INFO_TITLE_FLASHBACK" body="MORE_INFO_BODY_FLASHBACK" zone="ZONE_ANY" />'),
	(r'^Replicate\W', '<HELP title="MORE_INFO_TITLE_REPLICATE" body="MORE_INFO_BODY_REPLICATE" zone="ZONE_ANY" />')
])

dicPatternCosts = dict([
	('^' + SimpleCostPattern() + '(?:: |, )', '<COST mana_cost="@1@" type="Mana" />'),
	('^(Flashback|Replicate|Equip|Outlast)\W' + SimpleCostPattern() + '$', '<COST mana_cost="@2@" type="Mana" />'),
	('(?:^S|, S|, s)acrifice ENGLISHCARDNAME(: |, )', '<COST type="SacrificeSelf" />'),
	('(?:^P|, P|, p)ay ([0-9]+) life(?:: |, )', '<COST type="Life" amount="@1@" />'),
	('(?:^|, ){T}(?:: |, )', '<COST type="TapSelf" />'),
	('(?:^|, ){q}(?:: |, )', '<COST type="UntapSelf" />'),
	('\+(\d+):',
r"""<AVAILABILITY>
			return PLW_MyPlayerDC( EffectController() ):Get_Int(PLW_TEFERI_ARCHMAGE_EMBLEM) == 1 or EffectController():IsSorceryTime()
		</AVAILABILITY>
		<COST type="generic">
			<PREREQUISITE>
				return true
			</PREREQUISITE>
			<RESOLUTION_TIME_ACTION>
				if EffectSource() ~= nil then
					PLW_ShutDownDoublingSeason()
					EffectSource():AddCounters( MTG():GetCountersType("Loyalty"), @1@ )
				end
			</RESOLUTION_TIME_ACTION>
		</COST>"""),
	('-(\d+):',
r"""<AVAILABILITY>
			return PLW_MyPlayerDC( EffectController() ):Get_Int(PLW_TEFERI_ARCHMAGE_EMBLEM) == 1 or EffectController():IsSorceryTime() == true
		</AVAILABILITY>
		<COST type="RemoveCountersSelf" amount="@1@" counter_type="Loyalty" />"""),
	('(?:^|, )Remove ' + NumberWordPattern() + ' ' + '(' + SimpleCounterPattern(True) + '|(?:[a-z]+))' + ' counter(?:s|) from ENGLISHCARDNAME(?:: |, )', '<COST type="RemoveCountersSelf" amount="#1#" counter_type="#2#" />'),

	('(?:^S|, S|, s)acrifice ' + NumberWordPattern() + ' creature(?:|s)(?:: |, )',
r"""<COST type="Sacrifice" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CREATURE_TO_SACRIFICE" item_count="#1#" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</COST_DEFINITION>"""),

	('(?:^S|, S|, s)acrifice another creature(?:: |, )',
r"""<COST type="Sacrifice" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CREATURE_TO_SACRIFICE" item_count="1" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
			oFilter:Add(FE_INSTANCE, OP_NOT, EffectSource())
		</COST_DEFINITION>"""),

	('(?:^D|, D|, d)iscard ' + NumberWordPattern() + ' card(?:|s)(?:: |, )',
r"""<COST type="Discard" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CARD_TO_DISCARD" item_count="#1#" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:SetZone(ZONE_HAND, EffectController())
		</COST_DEFINITION>"""),

	('(?:^D|, D|, d)iscard ' + NumberWordPattern() + ' creature card(?:|s)(?:: |, )',
r"""<COST type="Discard" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CARD_TO_DISCARD" item_count="#1#" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:SetZone(ZONE_HAND, EffectController())
			oFilter:Add(FE_CARD_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</COST_DEFINITION>"""),

	('(?:^D|, D|, d)iscard ' + NumberWordPattern() + ' land card(?:|s)(?:: |, )',
r"""<COST type="Discard" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CARD_TO_DISCARD" item_count="#1#" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:SetZone(ZONE_HAND, EffectController())
			oFilter:Add(FE_CARD_TYPE, OP_IS, CARD_TYPE_LAND)
		</COST_DEFINITION>"""),

	('(?:^D|, D|, d)iscard ' + NumberWordPattern() + ' artifact card(?:|s)(?:: |, )',
r"""<COST type="Discard" definition="0" compartment="1" query_tag="CARD_QUERY_CHOOSE_CARD_TO_DISCARD" item_count="#1#" />
		<COST_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:SetZone(ZONE_HAND, EffectController())
			oFilter:Add(FE_CARD_TYPE, OP_IS, CARD_TYPE_ARTIFACT)
		</COST_DEFINITION>"""),
		
		(r'^(?:\||)Cohort(?:\||) — {T}, Tap an untapped Ally you control(?:: |, )',
r"""<COST type="TapSelf" />
		<COST type="Tap" definition="0" compartment="0" query_tag="CARD_QUERY_CHOOSE_CREATURE_TAP" item_count="1" />
		<COST_DEFINITION>
			local oFilter = ClearFilter()
			oFilter:Add(FE_IS_TAPPED, false)
			oFilter:Add(FE_SUBTYPE, OP_IS, CREATURE_TYPE_ALLY)
			oFilter:Add(FE_CONTROLLER, OP_IS, EffectController())
			oFilter:Add(FE_CARD_INSTANCE, OP_NOT, EffectSource())
		</COST_DEFINITION>""")
])

dicPatternTriggers = dict([

	(r'^At the beginning of each (?:upkeep|player’s upkeep),',
r"""<TRIGGER value="BEGINNING_OF_STEP">
			return MTG():GetStep() == STEP_UPKEEP
		</TRIGGER>"""),

	(r'^At the beginning of your upkeep,',
r"""<TRIGGER value="BEGINNING_OF_PLAYERS_STEP" simple_qualifier="controller">
			return MTG():GetStep() == STEP_UPKEEP
		</TRIGGER>"""),

	(r'^When ENGLISHCARDNAME enters the battlefield,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" />"""),

	(r'^When ENGLISHCARDNAME leaves the battlefield,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="self" to_zone="ZONE_ANY" from_zone="ZONE_BATTLEFIELD" />"""),

	(r'^When ENGLISHCARDNAME dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="self" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD" />"""),

	(r'^Whenever ENGLISHCARDNAME deals damage to an opponent,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" simple_qualifier="self" damage_type="all">
			return SecondaryPlayer() ~= nil and EffectController() ~= nil and SecondaryPlayer():GetTeam() ~= EffectController():GetTeam()
		</TRIGGER>"""),

	(r'^Whenever ENGLISHCARDNAME deals damage,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE" simple_qualifier="self" damage_type="all" />"""),

	(r'^Whenever ENGLISHCARDNAME deals combat damage to a player,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" simple_qualifier="self" damage_type="combat" />"""),

	(r'^Whenever a creature deals combat damage to you,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" damage_type="combat">
			return SecondaryPlayer() ~= nil and EffectController() ~= nil and SecondaryPlayer() == EffectController() and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever ENGLISHCARDNAME attacks,',
r"""<TRIGGER value="ATTACKING" simple_qualifier="self" />"""),

	(r'^Whenever ENGLISHCARDNAME blocks a creature,',
r"""<TRIGGER value="BLOCKS_A_CREATURE" simple_qualifier="self" />"""),

	(r'^Whenever ENGLISHCARDNAME becomes blocked(?:| by a creature),',
r"""<TRIGGER value="WAS_BLOCKED_BY_A_CREATURE" simple_qualifier="self" />"""),

	(r'^Whenever another creature dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="another" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever a creature dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever a creature you control dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="objectyoucontrol" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever a nontoken creature you control dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" simple_qualifier="objectyoucontrol" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE) and TriggerObject():IsToken() == false
		</TRIGGER>"""),

	(r'^Whenever a creature an opponent controls dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and EffectController() ~= nil and TriggerObject():GetPlayer():GetTeam() ~= EffectController():GetTeam() and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever another permanent enters the battlefield under your control,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject():GetCardType():IsPermanent() and TriggerObject() ~= EffectSource()
		</TRIGGER>"""),

	(r'^Whenever a creature enters the battlefield under your control,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever another creature enters the battlefield,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="another" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever another creature enters the battlefield under your control,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE) and TriggerObject() ~= EffectSource()
		</TRIGGER>"""),

	(r'^Whenever another creature leaves the battlefield,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" to_zone="ZONE_ANY" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE) and TriggerObject() ~= EffectSource()
		</TRIGGER>"""),

	(r'^Whenever a player casts a spell,',
r"""<TRIGGER value="SPELL_PLAYED" />"""),

	(r'^Whenever you cast a creature spell,',
r"""<TRIGGER value="SPELL_PLAYED" simple_qualifier="controller">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever you cast a noncreature spell,',
r"""<TRIGGER value="SPELL_PLAYED" simple_qualifier="controller">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE) == false
		</TRIGGER>"""),

	(r'^Whenever you cast an enchantment spell,',
r"""<TRIGGER value="SPELL_PLAYED" simple_qualifier="controller">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_ENCHANTMENT)
		</TRIGGER>"""),

	(r'^Whenever you cast an artifact spell,',
r"""<TRIGGER value="SPELL_PLAYED" simple_qualifier="controller">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_ARTIFACT)
		</TRIGGER>"""),

	(r'^Whenever equipped creature dies,',
r"""<TRIGGER value="ZONECHANGE_BEGIN" to_zone="ZONE_GRAVEYARD" from_zone="ZONE_BATTLEFIELD">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent()
		</TRIGGER>"""),

	(r'^Whenever equipped creature attacks,',
r"""<TRIGGER value="ATTACKING">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent()
		</TRIGGER>"""),

	(r'^Whenever equipped creature deals combat damage,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE" damage_type="combat">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent()
		</TRIGGER>"""),

	(r'^Whenever equipped creature deals damage to a player,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" damage_type="all">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent()
		</TRIGGER>"""),

	(r'^Whenever equipped creature deals combat damage to a player,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_PLAYER" damage_type="combat">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent()
		</TRIGGER>"""),

	(r'^Whenever equipped creature deals comat damage to a creature,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_OBJECT" damage_type="combat">
			return TriggerObject() ~= nil and EffectSource() ~= nil and TriggerObject() == EffectSource():GetParent() and SecondaryObject():GetCardType():Test( CARD_TYPE_CREATURE )
		</TRIGGER>"""),

	(r'^Whenever equipped creature deals damage to a creature,',
r"""<TRIGGER value="SOURCE_DEALS_DAMAGE_TO_OBJECT" damage_type="all">
			return TriggerObject() ~= nil and EffectSource() ~= nil and Secondary Object ~= nil and TriggerObject() == EffectSource():GetParent() and SecondaryObject():GetCardType():Test( CARD_TYPE_CREATURE )
		</TRIGGER>"""),

	(r'^(?:\||)Landfall(?:\||) — Whenever a land enters the battlefield under your control,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject() ~= nil and TriggerObject():GetCardType():Test(CARD_TYPE_LAND) 
		</TRIGGER>"""),

	(r'^(?:\||)Imprint(?:\||) — When ENGLISHCARDNAME enters the battlefield,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY" />"""),

	(r'^(?:\||)Rally(?:\||) — When ENGLISHCARDNAME or another Ally enters the battlefield under your control,',
r"""<TRIGGER value="ZONECHANGE_END" simple_qualifier="self" to_zone="ZONE_BATTLEFIELD" />
		<TRIGGER value="ZONECHANGE_END" simple_qualifier="objectyoucontrol" to_zone="ZONE_BATTLEFIELD" from_zone="ZONE_ANY">
			return TriggerObject():GetSubType():Test(CREATURE_TYPE_ALLY) and TriggerObject() ~= EffectSource()
		</TRIGGER>"""),

	(r'^Whenever a creature you control becomes renowned,',
r"""<TRIGGER value="PHENOMENON_ENCOUNTERED" simple_qualifier="objectyoucontrol">
			return TriggerObject() ~= nil and RSN_GetObjectDC(TriggerObject()) ~= nil and RSN_GetObjectDC(TriggerObject()):Get_Int(CW_TRIGGER) == CW_TRIGGER_BECAME_RENOWNED and TriggerObject():GetCardType():Test(CARD_TYPE_CREATURE)
		</TRIGGER>"""),

	(r'^Whenever ENGLISHCARDNAME becomes renowned,',
r"""<TRIGGER value="PHENOMENON_ENCOUNTERED" simple_qualifier="self">
			return TriggerObject() ~= nil and RSN_GetObjectDC(TriggerObject()) ~= nil and RSN_GetObjectDC(TriggerObject()):Get_Int(CW_TRIGGER) == CW_TRIGGER_BECAME_RENOWNED
		</TRIGGER>"""),
	
	(r'^Whenever you gain life,',
r"""<TRIGGER value="PLAYER_GAINED_LIFE" simple_qualifier="controller" />"""),

	(r'^Whenever ENGLISHCARDNAME attacks or blocks,',
r"""<TRIGGER value="ATTACKING" simple_qualifier="self" />
		<TRIGGER value="BLOCKING" simple_qualifier="self" />"""),
	
	(r'^Whenever ENGLISHCARDNAME crews a Vehicle,',
r"""<TRIGGER value="PHENOMENON_ENCOUNTERED" simple_qualifier="self">
			return TriggerObject() ~= nil and RSN_GetObjectDC(TriggerObject()) ~= nil and RSN_GetObjectDC(TriggerObject()):Get_Int(CW_TRIGGER) == CW_TRIGGER_CREWED_VEHICLE
		</TRIGGER>""")
])

dicPatternEffects = dict([

	(r'^Enchant creature$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_ENCHANT" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			local oSource = EffectSource()
			if oTarget ~= nil and oSource ~= nil then
				oSource:Attach(oTarget)
			end
		</RESOLUTION_TIME_ACTION>
		<AI_SIMPLIFIED_TARGETING compartment="0" hint="HINT_NEUTRAL" />"""),

	(r'Equip ' + SimpleCostPattern() + '$',
r"""<AVAILABILITY sorcery_time="1" />
		<AI_AVAILABILITY type="restriction" restriction_type="equip" />
		<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_EQUIP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
			oFilter:Add(FE_CONTROLLER, OP_IS, EffectController())
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			local oSource = EffectSource()
			if oTarget ~= nil and oSource ~= nil then
				oSource:Attach(oTarget)	
			end
		</RESOLUTION_TIME_ACTION>
		<AUTO_SKIP>
			return EffectDC():Get_Targets(0):Get_CardPtr(0) == nil
		</AUTO_SKIP>"""),

	(r': Regenerate ENGLISHCARDNAME.$',
r"""<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:GiveRegeneration()
			end
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="declare_blockers" type="window" window_in_combat="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="restriction" restriction_type="no_regeneration_shield" />
		<AI_MODIFIERS combat_immunity="1" />"""),
		
	(r'(?:^B|: B|, b)olster ([1-9]).$',
r"""<RESOLUTION_TIME_ACTION>
			local i = 0
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
			oFilter:Add(FE_CONTROLLER, OP_IS, EffectController())
			if oFilter:CountStopAt(1) == 1 then
				local oSubfilter = oFilter:AddSubFilter_Or()
				while true do
					oSubfilter:Add(FE_TOUGHNESS, OP_EQUALS, i )
					if oFilter:CountStopAt(@1@-1) == @1@-1 then
						EffectController():ChooseItem("CARD_QUERY_CHOOSE_A_CREATURE_TO_GET_@1@_PLUS1_PLUS1_COUNTERS", EffectDC():Make_Targets(0))
						return
					elseif oFilter:CountStopAt(@1@) == 1 then
						oFilter:EvaluateObjects()
						EffectDC():Make_Targets(0):Set_CardPtr(0, oFilter:GetNthEvaluatedObject(0))
						return
					end
					i = i + 1
				end
			end
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:AddCounters(MTG():PlusOnePlusOneCounters(), @1@)
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^S|: S|, s)upport ([1-9]).$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_GET_PLUS1_PLUS1_COUNTER" definition="0" compartment="0" count="@1@" up_to="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
			oFilter:Add(FE_CARD_INSTANCE, OP_NOT, EffectSource())
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTargets = EffectDC():Get_Targets(0)
			if oTargets ~= nil then
				for i=0,@1@-1 do
					local oTarget = oTargets:Get_CardPtr(i)
					if oTarget ~= nil then
						oTarget:AddCounters(MTG():PlusOnePlusOneCounters(), 1)
					end
				end
			end
		</RESOLUTION_TIME_ACTION>
		<AI_SIMPLIFIED_TARGETING compartment="0" hint="HINT_ALLIED_ONLY" />"""),
	
	(r'(?:^D|, d|: D)raw ' + NumberWordPattern() + ' card(?:|s).$',
r"""<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			if oController ~= nil then
				oController:DrawCards(#1#)
			end
		</RESOLUTION_TIME_ACTION>"""),
	
	(r'(?:^Y|: Y|, y)ou (gain|lose) ([1-9]) life.$',
r"""<RESOLUTION_TIME_ACTION>
			local oController = EffectController()
			if oController ~= nil then
				oController:#1#Life(@2@)
			end
		</RESOLUTION_TIME_ACTION>"""),
	
	(r'(?:: P|, p)ut ' + NumberWordPattern() + ' ' + SimpleCounterPattern() + ' counter(?:|s) on (?:ENGLISHCARDNAME|it).$',
r"""<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(#2#, #1#)
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:: P|, p)ut ' + NumberWordPattern() + ' ([a-z]+) counter(?:|s) on ENGLISHCARDNAME.$',
r"""<COUNTER_REGISTRATION name="#2#" proliferate="11" />
		<RESOLUTION_TIME_ACTION>
			local oSource = EffectSource()
			if oSource ~= nil then
				oSource:AddCounters(MTG():GetCountersType("#2#"), #1#)
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^S|: S|, s)cry ([1-9]).$',
r"""<RESOLUTION_TIME_ACTION>
			TFM_Scry_Resolution1(@1@)
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			TFM_Scry_Resolution2()
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			TFM_Scry_Resolution3()
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target permanent.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_PERMANENT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_IS_PERMANENT, true)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target artifact.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ARTIFACT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ARTIFACT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target creature.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target enchantment.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ENCHANTMENT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ENCHANTMENT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target land.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_LAND_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_LAND)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target planeswalker.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_PLANESWALKER_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_PLANESWALKER)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Destroy()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target permanent. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_PERMANENT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_IS_PERMANENT, true)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target artifact. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ARTIFACT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ARTIFACT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target creature. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target enchantment. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ENCHANTMENT_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ENCHANTMENT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target land. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_LAND_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_LAND)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^D|: D|, d)estroy target planeswalker. It can’t be regenerated.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_PLANESWALKER_TO_DESTROY" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_PLANESWALKER)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:DestroyWithoutRegenerate()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^T|: T|, t)ap target permanent.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_PERMANENT_TO_TAP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_IS_PERMANENT, true)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Tap()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^T|: T|, t)ap target artifact.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ARTIFACT_TO_TAP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ARTIFACT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Tap()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^T|: T|, t)ap target creature.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_CREATURE_TO_TAP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Tap()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^T|: T|, t)ap target enchantment.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_ENCHANTMENT_TO_TAP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_ENCHANTMENT)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Tap()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^T|: T|, t)ap target land.$',
r"""<TARGET tag="CARD_QUERY_CHOOSE_LAND_TO_TAP" definition="0" compartment="0" count="1" />
		<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_LAND)
		</TARGET_DEFINITION>
		<RESOLUTION_TIME_ACTION>
			local oTarget = EffectDC():Get_Targets(0):Get_CardPtr(0)
			if oTarget ~= nil then
				oTarget:Tap()
			end
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^E|: E|, E)NGLISHCARDNAME deals ([0-9]+) damage to target creature or player.$',
r"""<TARGET_DEFINITION id="0">
			local oFilter = ClearFilter()
			oFilter:SetFilterType(FILTER_TYPE_CARDS + FILTER_TYPE_PLAYERS)
			oFilter:Add(FE_TYPE, OP_IS, CARD_TYPE_CREATURE)
		</TARGET_DEFINITION>
		<TARGET tag="CARD_QUERY_CHOOSE_DEAL_@1@_DAMAGE" definition="0" compartment="0" count="1" />
		<RESOLUTION_TIME_ACTION>
			local oCreature = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_CardPtr(0)
			local oPlayer = EffectDC():Get_Targets(0) and EffectDC():Get_Targets(0):Get_PlayerPtr(0)
			if oCreature ~= nil then
				EffectSourceLKI():DealDamageTo(#1#, oCreature)
			elseif oPlayer ~= nil then
				EffectSourceLKI():DealDamageTo(#1#, oPlayer)
			end
		</RESOLUTION_TIME_ACTION>
		<AI_SIMPLIFIED_TARGETING compartment="0" hint="HINT_ENEMY_ONLY" />"""),
		
		(r'(?:^E|: E|, E)NGLISHCARDNAME gets (\+|\-)([0-9])/(\+|\-)([0-9]) until end of turn.$',
r"""<CONTINUOUS_ACTION layer="7C">
			if EffectSource() ~= nil then
				local oCharacteristics = EffectSource():GetCurrentCharacteristics()
				oCharacteristics:Power_Add(#1#@2@)
				oCharacteristics:Toughness_Add(#3#@4@)
			end
		</CONTINUOUS_ACTION>
		<DURATION simple_duration="UntilEOT" />"""),
		
		(r'(?:^I|: I|, i)nvestigate.$',
r"""<RESOLUTION_TIME_ACTION>
			CW_General_FireTrigger(CW_TRIGGER_INVESTIGATE, EffectSource():GetController())
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			CW_Tokens("CLUE_A_C_S")
		</RESOLUTION_TIME_ACTION>"""),
		
		(r'(?:^A|: A)dd \{([WUBRG]|[0-9]+)\} to your mana pool.$',
r"""<PLAY_TIME_ACTION>
			RSN_MarkManaAbilityStart()
		</PLAY_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_Produce( "{@1@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_EliminateExtraManaTokens()
			S_DisplayManaPool(EffectController())
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="upkeep" type="window" />
		<AI_AVAILABILITY window_step="main_1" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="begin_combat" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_attackers" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_blockers" type="window" />
		<AI_AVAILABILITY window_step="main_2" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY type="in_response" response_source="1" response_target="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="in_response" response_target="1" />
<!--SIMPLEMANAPRODUCTION-@1@-->"""),
		
		(r'(?:^A|: A)dd \{([WUBRG]|[0-9]+)\} or \{([WUBRG]|[0-9]+)\} to your mana pool.$',
r"""<MODE_SELECT tag="MODE_CHOOSE_ONE">
			<MODE tag="RSN_MODE_PRODUCE_@1@" index="1" />
			<MODE tag="RSN_MODE_PRODUCE_@2@" index="2" />
		</MODE_SELECT>
		<PLAY_TIME_ACTION>
			RSN_MarkManaAbilityStart()
		</PLAY_TIME_ACTION>
		<RESOLUTION_TIME_ACTION mode="1">
			RSN_Produce( "{@1@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION mode="2">
			RSN_Produce( "{@2@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_EliminateExtraManaTokens()
			S_DisplayManaPool(EffectController())
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="upkeep" type="window" />
		<AI_AVAILABILITY window_step="main_1" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="begin_combat" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_attackers" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_blockers" type="window" />
		<AI_AVAILABILITY window_step="main_2" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY type="in_response" response_source="1" response_target="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="in_response" response_target="1" />
<!--SIMPLEMANAPRODUCTION-@1@-@2@-->"""),
		
		(r'(?:^A|: A)dd \{([WUBRG]|[0-9]+)\}, \{([WUBRG]|[0-9]+)\}, or \{([WUBRG]|[0-9]+)\} to your mana pool.$',
r"""<MODE_SELECT tag="MODE_CHOOSE_ONE">
			<MODE tag="RSN_MODE_PRODUCE_@1@" index="1" />
			<MODE tag="RSN_MODE_PRODUCE_@2@" index="2" />
			<MODE tag="RSN_MODE_PRODUCE_@3@" index="3" />
		</MODE_SELECT>
		<PLAY_TIME_ACTION>
			RSN_MarkManaAbilityStart()
		</PLAY_TIME_ACTION>
		<RESOLUTION_TIME_ACTION mode="1">
			RSN_Produce( "{@1@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION mode="2">
			RSN_Produce( "{@2@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION mode="3">
			RSN_Produce( "{@3@}", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_EliminateExtraManaTokens()
			S_DisplayManaPool(EffectController())
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="upkeep" type="window" />
		<AI_AVAILABILITY window_step="main_1" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="begin_combat" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_attackers" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_blockers" type="window" />
		<AI_AVAILABILITY window_step="main_2" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY type="in_response" response_source="1" response_target="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="in_response" response_target="1" />
<!--SIMPLEMANAPRODUCTION-@1@-@2@-@3@-->"""),
		
		(r'(?:^A|: A)dd one mana of any color to your mana pool.$',
r"""<PLAY_TIME_ACTION>
			RSN_MarkManaAbilityStart()
			local oPlayer = EffectController()
			local oCard = EffectSource()
			if (oPlayer ~= nil) then
				oPlayer:BeginNewMultipleChoice()
					oPlayer:AddMultipleChoiceAnswer( "RSN_MODE_PRODUCE_W" )
					oPlayer:AddMultipleChoiceAnswer( "RSN_MODE_PRODUCE_U" )
					oPlayer:AddMultipleChoiceAnswer( "RSN_MODE_PRODUCE_B" )
					oPlayer:AddMultipleChoiceAnswer( "RSN_MODE_PRODUCE_R" )
					oPlayer:AddMultipleChoiceAnswer( "RSN_MODE_PRODUCE_G" )
				oPlayer:AskMultipleChoiceQuestion( "MODE_CHOOSE_ONE", oCard )
			end
		</PLAY_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			CW_IMM_Produce(EffectController():GetMultipleChoiceResult() + 1)
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_EliminateExtraManaTokens()
			S_DisplayManaPool(EffectController())
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="upkeep" type="window" />
		<AI_AVAILABILITY window_step="main_1" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="begin_combat" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_attackers" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_blockers" type="window" />
		<AI_AVAILABILITY window_step="main_2" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY type="in_response" response_source="1" response_target="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="in_response" response_target="1" />
<!--SIMPLEMANAPRODUCTION-W-U-B-R-G-->"""),
		
		(r'(?:^A|: A)dd ' + SimpleCostPattern() + SimpleCostPattern() + ' to your mana pool.$',
r"""<PLAY_TIME_ACTION>
			RSN_MarkManaAbilityStart()
		</PLAY_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_Produce( "@1@", 1 )
		</RESOLUTION_TIME_ACTION>
		<RESOLUTION_TIME_ACTION>
			RSN_EliminateExtraManaTokens()
			S_DisplayManaPool(EffectController())
			RSN_MarkManaAbilityEnd()
		</RESOLUTION_TIME_ACTION>
		<AI_AVAILABILITY window_step="upkeep" type="window" />
		<AI_AVAILABILITY window_step="main_1" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="begin_combat" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_attackers" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY window_step="declare_blockers" type="window" />
		<AI_AVAILABILITY window_step="main_2" window_turn="my_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" type="window" />
		<AI_AVAILABILITY window_step="end_of_turn" window_turn="their_turn" type="window" />
		<AI_AVAILABILITY type="in_response" response_source="1" response_target="1" />
		<AI_AVAILABILITY type="in_response" response_source="1" />
		<AI_AVAILABILITY type="in_response" response_target="1" />
<!--COMPLEXMANAPRODUCTION-#1#-->""")
])

dicPatternEffectsHelp = dict([
	(r': Regenerate ENGLISHCARDNAME.$', '<HELP title="MORE_INFO_TITLE_REGENERATE" body="MORE_INFO_BODY_REGENERATE" zone="ZONE_ANY" />'),
	(r'(^|: )Bolster [1-9].$', '<HELP title="CW_MORE_INFO_TITLE_BOLSTER" body="MORE_INFO_BODY_BOLSTER" zone="ZONE_ANY" />'),
	(r'(?:^S|: S|, s)upport ([1-9]).$', '<HELP title="CW_MORE_INFO_TITLE_SUPPORT" body="MORE_INFO_BODY_SUPPORT" zone="ZONE_ANY" />'),
	(r'(?:^I|: I|, i)nvestigate.$', '<HELP title="CW_MORE_INFO_TITLE_INVESTIGATE" body="MORE_INFO_BODY_INVESTIGATE" zone="ZONE_ANY" />')
])

dicCapitalization = dict([
	('A', r"""a"""),
	('B', r"""b"""),
	('C', r"""c"""),
	('D', r"""d"""),
	('F', r"""e"""),
	('F', r"""f"""),
	('G', r"""g"""),
	('H', r"""h"""),
	('I', r"""i"""),
	('J', r"""j"""),
	('K', r"""k"""),
	('L', r"""l"""),
	('M', r"""m"""),
	('N', r"""n"""),
	('O', r"""o"""),
	('P', r"""p"""),
	('Q', r"""q"""),
	('R', r"""r"""),
	('S', r"""s"""),
	('T', r"""t"""),
	('U', r"""u"""),
	('V', r"""v"""),
	('W', r"""w"""),
	('X', r"""x"""),
	('Y', r"""y"""),
	('Z', r"""z"""),
	('Ä', r"""ä"""),
	('Ö', r"""ö"""),
	('Ü', r"""ü"""),
	('À', r"""à"""),
	('Â', r"""â"""),
	('Æ', r"""æ"""),
	('Ç', r"""ç"""),
	('É', r"""é"""),
	('È', r"""è"""),
	('Ê', r"""ê"""),
	('Ë', r"""ë"""),
	('Î', r"""î"""),
	('Ï', r"""ï"""),
	('Ô', r"""ô"""),
	('Œ', r"""œ"""),
	('Ù', r"""ù"""),
	('Û', r"""û"""),
	('Ÿ', r"""ÿ"""),
	('Ì', r"""ì"""),
	('Ò', r"""ò"""),
	('Ó', r"""ó""")
])

lstLanguageShort = ['en-US', 'es-ES', 'fr-FR', 'it-IT', 'pt-BR']


# Throughout the code, you'll see this code:
# for iIndex in range(1, len(moMatch.groups())+1):
#	 sTemplate = sTemplate.replace('#' + str(iIndex) + '#', AutoReplacePatternWords(str(moMatch.group(iIndex)))).replace('@' + str(iIndex) + '@', str(moMatch.group(iIndex)))
# TFM originally made this, but I added the first replace and contained function. I also corrected the SimpleCostPattern() function to be more reliable.
# In the ability text you'll match "You gain 3 life." You replace this with text which will match many versions of the same ability.
# "(?:^Y|: Y|, y)ou (gain|lose) ([0-9]+) life.$"
# Anything in () can be referenced later by an index number unless the first two characters in the () are ?:.
# So when you need to allow multiple possibilities but don't care which was the case, use ?:.
# In the above, you get:
# 1: 'gain'
# 2: '3'
# In the ability's code, you use @1@ when you need to use the first result unaltered.
# Use #1# when you need to use the first result but it needs converted first.
# The code for this ability is this:
#	<RESOLUTION_TIME_ACTION>
#		local EffectController = EffectController()
#		if EffectController ~= nil then
#			EffectController:#1#Life(@2@)
#		end
#	</RESOLUTION_TIME_ACTION>
# So, @2@ is replaced by the second result without being altered.
# #1# is replaced by the first result, but it's first run through AutoReplacePatternWords().
# Add any conversions to AutoReplacePatternWords() and they should function without any further work.
# +1/+1 returns MTG():PlusOnePlusOneCounter(). Thirteen returns 13.
# In other words, it's a simple translator that translates from normal speech to code equivilants.
# You'll notice that "gain" and "lose" aren't in the list. That's because they don't need to be.
# If a word isn't found, it returns it after capitalizing the first letter, which is all that's needed for "gain", "lose", and custom counters.
# So, it looks up "gain" and doesn't find it, returning "Gain", instead. It doesn't look up "3", it just uses it as is, resulting in "EffectController:GainLife(3)".

# 1.10
# 06-09-16 - Replaced variables named after functions to not have the same name.
# 10-09-16 - Fixed triggered ability guessing.
# 10-09-16 - Fixed token creation. Namely, fixed first strike (was converting to "FS" instead of "F"), double strike ("DS" -> "D"), and blue ("B" -> "U") tokens.


# 22-09-16 - Added Emerge.
# 22-09-16 - Added Fabricate.
# 22-09-16 - Added Crew.
# 22-09-16 - Added Pilot triggers.

# 1.11
# 23-09-16 - Changed Crew to use functions.
# 23-09-16 - Added support for "Create" keyword based token creation.

# 1.12
# 23-06-17 - Added Embalm.
# 23-06-17 - Added Eternalize.
# 23-06-17 - Added Afflict.
# 23-06-17 - Corrected issue where needing to register multiple types of tokens would skip all but the first.

# 1.13
# 24-06-17 - Removed MID from appearing in the FILENAME.
# 24-06-17 - Added Loyalty support, including initial value and cost.

# 1.13.1
# 30-06-17 - Added embalm and eternalize triggers and characteristics.